@@ -12,6 +12,8 @@
 
 // LCOV_BR_EXCL_START
 // LCOV_EXCL_START
+//
+struct empty {};
 
 NS_ASSUME_NONNULL_BEGIN
 
@@ -147,8 +149,25 @@ PyErr_Clear();
 
 END_UNITTEST
 
+BEGIN_UNITTEST(BitFieldSize)
+struct bitfield1 {
+    unsigned int first : 4;
+};
+struct bitfield2 {
+    unsigned int first : 4;
+    unsigned int second : 30;
+};
+
+ASSERT_EQUALS(sizeof(struct bitfield1), PyObjCRT_SizeOfType(@encode(struct bitfield1)),
+              "%d");
+ASSERT_EQUALS(sizeof(struct bitfield2), PyObjCRT_SizeOfType(@encode(struct bitfield2)),
+              "%d");
+END_UNITTEST
+
 BEGIN_UNITTEST(StructSize)
 
+ASSERT_EQUALS(sizeof(struct empty), PyObjCRT_SizeOfType(@encode(struct empty)), "%d");
+
 ASSERT_EQUALS(sizeof(struct Struct1), PyObjCRT_SizeOfType(@encode(struct Struct1)), "%d");
 
 ASSERT_EQUALS(sizeof(struct Struct2), PyObjCRT_SizeOfType(@encode(struct Struct2)), "%d");
@@ -159,10 +178,17 @@ ASSERT_EQUALS(sizeof(struct Struct4), PyObjCRT_SizeOfType(@encode(struct Struct4
 
 ASSERT_EQUALS(sizeof(NSRect), PyObjCRT_SizeOfType(@encode(NSRect)), "%d");
 
+Py_ssize_t r = PyObjCRT_SizeOfType("{p=!}");
+FAIL_IF(r != -1);
+PyErr_Clear();
+
 END_UNITTEST
 
 BEGIN_UNITTEST(StructAlign)
 
+ASSERT_EQUALS(__alignof__(struct empty), PyObjCRT_AlignOfType(@encode(struct empty)),
+              "%d");
+
 ASSERT_EQUALS(__alignof__(struct Struct1), PyObjCRT_AlignOfType(@encode(struct Struct1)),
               "%d");
 
@@ -175,6 +201,16 @@ ASSERT_EQUALS(__alignof__(struct Struct3), PyObjCRT_AlignOfType(@encode(struct S
 ASSERT_EQUALS(__alignof__(struct Struct4), PyObjCRT_AlignOfType(@encode(struct Struct4)),
               "%d");
 
+ASSERT_EQUALS(__alignof__(struct empty), PyObjCRT_AlignOfType("n{empty=}"), "%d");
+
+Py_ssize_t r = PyObjCRT_AlignOfType("{p=!");
+FAIL_IF(r != -1);
+PyErr_Clear();
+
+r = PyObjCRT_AlignOfType("(p=!");
+FAIL_IF(r != -1);
+PyErr_Clear();
+
 END_UNITTEST
 
 BEGIN_UNITTEST(FillStruct1)
@@ -183,12 +219,9 @@ PyObject*      input;
 struct Struct1 output;
 int            r;
 
-input = PyTuple_New(2);
+input = PyTuple_Pack(2, PyLong_FromLong(1), PyFloat_FromDouble(2));
 FAIL_IF(input == NULL);
 
-PyTuple_SetItem(input, 0, PyLong_FromLong(1));
-PyTuple_SetItem(input, 1, PyFloat_FromDouble(2));
-
 r = depythonify_c_value(@encode(struct Struct1), input, &output);
 FAIL_IF(r < 0);
 
@@ -206,20 +239,14 @@ PyObject*      v;
 struct Struct2 output;
 int            r;
 
-input = PyTuple_New(3);
+v = PyTuple_Pack(5, PyLong_FromLong(10), PyLong_FromLong(11), PyLong_FromLong(12),
+                 PyLong_FromLong(13), PyLong_FromLong(14));
+FAIL_IF(v == NULL);
+
+input = PyTuple_Pack(3, PyLong_FromLong(1), PyFloat_FromDouble(2), v);
+Py_CLEAR(v);
 FAIL_IF(input == NULL);
 
-v = PyTuple_New(5);
-PyTuple_SetItem(v, 0, PyLong_FromLong(10));
-PyTuple_SetItem(v, 1, PyLong_FromLong(11));
-PyTuple_SetItem(v, 2, PyLong_FromLong(12));
-PyTuple_SetItem(v, 3, PyLong_FromLong(13));
-PyTuple_SetItem(v, 4, PyLong_FromLong(14));
-
-PyTuple_SetItem(input, 0, PyLong_FromLong(1));
-PyTuple_SetItem(input, 1, PyFloat_FromDouble(2));
-PyTuple_SetItem(input, 2, v);
-
 r = depythonify_c_value(@encode(struct Struct2), input, &output);
 FAIL_IF(r < 0);
 
@@ -241,12 +268,9 @@ PyObject*      input;
 struct Struct3 output;
 int            r;
 
-input = PyTuple_New(2);
+input = PyTuple_Pack(2, PyBytes_FromStringAndSize("\001", 1), PyLong_FromLong(2));
 FAIL_IF(input == NULL);
 
-PyTuple_SetItem(input, 0, PyBytes_FromStringAndSize("\001", 1));
-PyTuple_SetItem(input, 1, PyLong_FromLong(2));
-
 r = depythonify_c_value(@encode(struct Struct3), input, &output);
 FAIL_IF(r < 0);
 
@@ -263,12 +287,9 @@ PyObject*      input;
 struct Struct4 output;
 int            r;
 
-input = PyTuple_New(2);
+input = PyTuple_Pack(2, PyBytes_FromStringAndSize("\001", 1), PyLong_FromLong(500000));
 FAIL_IF(input == NULL);
 
-PyTuple_SetItem(input, 0, PyBytes_FromStringAndSize("\001", 1));
-PyTuple_SetItem(input, 1, PyLong_FromLong(500000));
-
 r = depythonify_c_value(@encode(struct Struct4), input, &output);
 FAIL_IF(r < 0);
 
@@ -282,23 +303,21 @@ END_UNITTEST
 BEGIN_UNITTEST(FillStruct5Array)
 
 PyObject*    input;
-PyObject*    v;
+PyObject*    v1;
+PyObject*    v2;
 Struct5Array output;
 int          r;
 
-input = PyTuple_New(2);
+v1 = PyTuple_Pack(2, PyLong_FromLong(500000), PyBytes_FromStringAndSize("\001", 1));
+FAIL_IF(v1 == NULL);
+v2 = PyTuple_Pack(2, PyLong_FromLong(1000000), PyBytes_FromStringAndSize("\002", 1));
+FAIL_IF(v2 == NULL);
+
+input = PyTuple_Pack(2, v1, v2);
+Py_CLEAR(v1);
+Py_CLEAR(v2);
 FAIL_IF(input == NULL);
 
-v = PyTuple_New(2);
-PyTuple_SetItem(v, 0, PyLong_FromLong(500000));
-PyTuple_SetItem(v, 1, PyBytes_FromStringAndSize("\001", 1));
-PyTuple_SetItem(input, 0, v);
-
-v = PyTuple_New(2);
-PyTuple_SetItem(v, 0, PyLong_FromLong(1000000));
-PyTuple_SetItem(v, 1, PyBytes_FromStringAndSize("\002", 1));
-PyTuple_SetItem(input, 1, v);
-
 r = depythonify_c_value(@encode(Struct5Array), input, &output);
 FAIL_IF(r < 0);
 
@@ -722,20 +741,17 @@ int           r;
 output.before = 0xDEADBEEF;
 output.after  = 0xBEEFDEAD;
 
-input = PyTuple_New(2);
+v = PyTuple_Pack(2, PyLong_FromLong(10), PyLong_FromLong(11));
+FAIL_IF(v == NULL);
+
+t = PyTuple_Pack(2, PyLong_FromLong(20), PyLong_FromLong(21));
+FAIL_IF(t == NULL);
+
+input = PyTuple_Pack(2, v, t);
+Py_CLEAR(v);
+Py_CLEAR(t);
 FAIL_IF(input == NULL);
 
-v = PyTuple_New(2);
-PyTuple_SetItem(v, 0, PyLong_FromLong(10));
-PyTuple_SetItem(v, 1, PyLong_FromLong(11));
-
-t = PyTuple_New(2);
-PyTuple_SetItem(t, 0, PyLong_FromLong(20));
-PyTuple_SetItem(t, 1, PyLong_FromLong(21));
-
-PyTuple_SetItem(input, 0, v);
-PyTuple_SetItem(input, 1, t);
-
 r = depythonify_c_value(@encode(NSRect), input, &output.rect);
 FAIL_IF(r < 0);
 
@@ -828,6 +844,7 @@ END_UNITTEST
 BEGIN_UNITTEST(UnicodeFunctions)
 PyObject* unicode = PyUnicode_FromString("hello world");
 int       ok;
+ASSERT(unicode != NULL);
 
 ok = PyObjC_is_ascii_string(unicode, "hello world");
 ASSERT(ok);
@@ -853,6 +870,18 @@ ASSERT(ok);
 ok = PyObjC_is_ascii_prefix(unicode, "hello world!", 12);
 ASSERT(!ok);
 
+Py_CLEAR(unicode);
+
+unicode = PyUnicode_FromString("Straße");
+ASSERT(unicode != NULL);
+ok = PyObjC_is_ascii_prefix(unicode, "hello", 5);
+ASSERT(!ok);
+
+ok = PyObjC_is_ascii_string(unicode, "hello");
+ASSERT(!ok);
+
+Py_CLEAR(unicode);
+
 END_UNITTEST
 
 BEGIN_UNITTEST(DecimalSize)
@@ -963,7 +992,11 @@ END_UNITTEST
 
 BEGIN_UNITTEST(InvalidObjCPointer)
 
-/* Invalid type encoding */
+/* Invalid type encodings
+ *
+ * XXX: This tests are artificial, but at least
+ *      ensure that some error paths are hit.
+ */
 PyObject* p = PyObjCPointer_New(&p, "^{foo=");
 ASSERT(PyErr_Occurred());
 ASSERT(!p);
@@ -974,6 +1007,11 @@ ASSERT(PyErr_Occurred());
 ASSERT(!p);
 PyErr_Clear();
 
+p = PyObjCPointer_New(&p, "X");
+ASSERT(PyErr_Occurred());
+ASSERT(!p);
+PyErr_Clear();
+
 END_UNITTEST
 
 BEGIN_UNITTEST(InvalidRegistryUsage)
@@ -990,32 +1028,6 @@ PyErr_Clear();
 
 END_UNITTEST
 
-BEGIN_UNITTEST(BytesInterning)
-PyObject* b1;
-PyObject* b2;
-
-b1 = PyObjCBytes_InternFromString("hello");
-FAIL_IF(b1 == NULL);
-if (!PyBytes_Check(b1)) {
-    Py_DECREF(b1);
-    ASSERT(0);
-}
-ASSERT_EQUALS(strcmp(PyBytes_AsString(b1), "hello"), 0, "%d");
-
-b2 = PyObjCBytes_InternFromStringAndSize("hello world", 5);
-FAIL_IF(b2 == NULL);
-if (!PyBytes_Check(b2)) {
-    Py_DECREF(b2);
-    ASSERT(0);
-}
-
-ASSERT_EQUALS(b1, b2, "%p");
-
-Py_DECREF(b1);
-Py_DECREF(b2);
-
-END_UNITTEST
-
 BEGIN_UNITTEST(MethodSignatureString)
 PyObjCMethodSignature* sig = PyObjCMethodSignature_WithMetaData("@@:d", NULL, NO);
 FAIL_IF(sig == NULL);
@@ -1055,9 +1067,199 @@ FAIL_IF(!PyObjCRT_IsValidEncoding("n^{a=i}", 7));
 FAIL_IF(PyObjCRT_IsValidEncoding("n^{a=i}", 5));
 FAIL_IF(PyObjCRT_IsValidEncoding("n^{a=i}", 2));
 FAIL_IF(PyObjCRT_IsValidEncoding("n^{a=i}", 1));
+FAIL_IF(PyObjCRT_IsValidEncoding("n^{a=!}", 7));
 
 FAIL_IF(PyObjCRT_IsValidEncoding("{a=\"f\"i}", 8));
 
+FAIL_IF(!PyObjCRT_IsValidEncoding("<2f>", 4));
+FAIL_IF(PyObjCRT_IsValidEncoding("<2f", 3));
+FAIL_IF(PyObjCRT_IsValidEncoding("<2f>", 3));
+FAIL_IF(PyObjCRT_IsValidEncoding("<2ff>", 5));
+
+FAIL_IF(!PyObjCRT_IsValidEncoding("[2f]", 4));
+FAIL_IF(PyObjCRT_IsValidEncoding("[2!]", 4));
+
+FAIL_IF(PyObjCRT_IsValidEncoding("(p=ii)", 6));
+
+END_UNITTEST
+
+static int
+exception_text_contains(const char* text)
+{
+    PyObject* type;
+    PyObject* value;
+    PyObject* traceback;
+
+    PyErr_Fetch(&type, &value, &traceback);
+    if (type == NULL && value == NULL && traceback == NULL) {
+        /* No exception */
+        return 0;
+    }
+
+    PyErr_NormalizeException(&type, &value, &traceback);
+
+    PyObject* repr = PyObject_Repr(value);
+
+    Py_CLEAR(type);
+    Py_CLEAR(value);
+    Py_CLEAR(traceback);
+    if (repr == NULL) {
+        PyErr_Clear();
+        return 0;
+    }
+
+    PyObject* expected = PyUnicode_FromString(text);
+    if (expected == NULL) {
+        Py_CLEAR(repr);
+        PyErr_Clear();
+        return 0;
+    }
+
+    int r = PySequence_Contains(repr, expected);
+    if (r == -1) {
+        Py_CLEAR(repr);
+        Py_CLEAR(expected);
+        PyErr_Clear();
+        return 0;
+    }
+
+    if (r) {
+        Py_CLEAR(repr);
+        Py_CLEAR(expected);
+        return 1;
+    }
+
+    PyErr_Format(PyExc_ValueError, "%R does not contain %R", repr, expected);
+    Py_CLEAR(expected);
+    Py_CLEAR(repr);
+
+    return 0;
+}
+
+BEGIN_UNITTEST(CheckArgCount)
+/* C test because it hitting the error paths in regular tests
+ * is not possible.
+ *
+ * Also: currently only the core only * uses the this to check
+ * the number of arguments in functions with a fixed number
+ * of arguments.
+ */
+int       r;
+PyObject* callable = Py_None;
+
+r = PyObjC_CheckArgCount(callable, 1, 1, 1);
+FAIL_IF(r == -1);
+FAIL_IF(PyErr_Occurred());
+
+r = PyObjC_CheckArgCount(callable, 1, 3, 2);
+FAIL_IF(r == -1);
+FAIL_IF(PyErr_Occurred());
+
+r = PyObjC_CheckArgCount(callable, 1, 1, 2);
+FAIL_IF(r != -1);
+FAIL_IF(!exception_text_contains("None expected 1 arguments, got 2"));
+
+r = PyObjC_CheckArgCount(callable, 1, 3, 4);
+FAIL_IF(r != -1);
+FAIL_IF(!exception_text_contains("None expected between 1 and 3 arguments, got 4"));
+
+END_UNITTEST
+
+BEGIN_UNITTEST(NoKwNames)
+/* C test because it hitting the error paths in regular tests
+ * is not possible.
+ */
+int       r;
+PyObject* callable = Py_None;
+PyObject* kwnames;
+
+r = PyObjC_CheckNoKwnames(callable, NULL);
+FAIL_IF(r == -1);
+FAIL_IF(PyErr_Occurred());
+
+kwnames = PyList_New(0);
+if (kwnames == NULL) {
+    goto error;
+}
+
+r = PyObjC_CheckNoKwnames(callable, kwnames);
+Py_CLEAR(kwnames);
+FAIL_IF(r == -1);
+FAIL_IF(PyErr_Occurred());
+
+kwnames = PyList_New(0);
+if (kwnames == NULL) {
+    goto error;
+}
+if (PyList_Append(kwnames, Py_None) == -1) {
+    goto error;
+}
+
+r = PyObjC_CheckNoKwnames(callable, kwnames);
+Py_CLEAR(kwnames);
+FAIL_IF(r != -1);
+FAIL_IF(!exception_text_contains("None does not accept keyword arguments"));
+
+r = PyObjC_CheckNoKwnames(callable, callable);
+Py_CLEAR(kwnames);
+FAIL_IF(r != -1);
+PyErr_Clear();
+
+END_UNITTEST
+
+BEGIN_UNITTEST(PyObjC_NSMethodSignatureToTypeString_Errors)
+char               buffer[2048];
+char*              result;
+NSMethodSignature* sig = [NSURL
+    instanceMethodSignatureForSelector:@selector
+    (initByResolvingBookmarkData:options:relativeToURL:bookmarkDataIsStale:error:)];
+ASSERT(sig != nil);
+
+result = PyObjC_NSMethodSignatureToTypeString(sig, buffer, 0);
+ASSERT(result == NULL);
+ASSERT(PyErr_Occurred());
+PyErr_Clear();
+
+result = PyObjC_NSMethodSignatureToTypeString(sig, buffer, 4);
+ASSERT(result == NULL);
+ASSERT(PyErr_Occurred());
+PyErr_Clear();
+
+END_UNITTEST
+
+BEGIN_UNITTEST(SkippingTypeSpec)
+/* Some tests for 'PyObjCRT_SkipTypeSpec' that hit edge cases that
+ * aren't hit in regular code.
+ */
+const char* ts;
+const char* end;
+ts = "\"name\"id";
+
+end = PyObjCRT_SkipTypeSpec(ts);
+ASSERT(end != NULL);
+ASSERT(*end == 'i');
+ASSERT(!PyErr_Occurred());
+
+ts  = "@?<v@?@\"BAAssetPack\"@\"NSError\">24f";
+end = PyObjCRT_SkipTypeSpec(ts);
+ASSERT(end != NULL);
+ASSERT(*end == 'f');
+ASSERT(!PyErr_Occurred());
+
+ts  = "@?<v@?@\"BAAssetPack\"X\"NSError\">24f";
+end = PyObjCRT_SkipTypeSpec(ts);
+ASSERT(end == NULL);
+ASSERT(PyErr_Occurred());
+FAIL_IF(!exception_text_contains("Unhandled type"));
+PyErr_Clear();
+
+ts  = "@?<v@?@\"BAAssetPack\"@\"NSError\"";
+end = PyObjCRT_SkipTypeSpec(ts);
+ASSERT(end == NULL);
+ASSERT(PyErr_Occurred());
+FAIL_IF(!exception_text_contains("invalid block encoding"));
+PyErr_Clear();
+
 END_UNITTEST
 
 static PyMethodDef mod_methods[] = {TESTDEF(CheckNSInvoke),
@@ -1065,6 +1267,7 @@ static PyMethodDef mod_methods[] = {TESTDEF(CheckNSInvoke),
                                     TESTDEF(VectorSize),
                                     TESTDEF(VectorAlign),
                                     TESTDEF(StructSize),
+                                    TESTDEF(BitFieldSize),
                                     TESTDEF(StructAlign),
                                     TESTDEF(FillStruct1),
                                     TESTDEF(FillStruct2),
@@ -1096,9 +1299,12 @@ static PyMethodDef mod_methods[] = {TESTDEF(CheckNSInvoke),
                                     TESTDEF(AlwaysFails),
                                     TESTDEF(InvalidObjCPointer),
                                     TESTDEF(InvalidRegistryUsage),
-                                    TESTDEF(BytesInterning),
                                     TESTDEF(MethodSignatureString),
                                     TESTDEF(ValidEncoding),
+                                    TESTDEF(CheckArgCount),
+                                    TESTDEF(NoKwNames),
+                                    TESTDEF(PyObjC_NSMethodSignatureToTypeString_Errors),
+                                    TESTDEF(SkippingTypeSpec),
                                     {0, 0, 0, 0}};
 
 int
@@ -1112,16 +1318,25 @@ PyObjC_init_ctests(PyObject* m)
     PyMethodDef* cur;
     for (cur = mod_methods; cur->ml_name != NULL; cur++) {
         PyObject* meth = PyCFunction_NewEx(cur, NULL, NULL);
-        if (meth == NULL) {
-            Py_DECREF(d); // LCOV_EXCL_LINE
-            return -1;    // LCOV_EXCL_LINE
+        if (meth == NULL) { // LCOV_BR_EXCL_LINE
+            Py_DECREF(d);   // LCOV_EXCL_LINE
+            return -1;      // LCOV_EXCL_LINE
         }
-        if (PyDict_SetItemString(d, cur->ml_name, meth) < 0) {
+
+        PyObject* key = PyUnicode_FromString(cur->ml_name);
+        if (key == NULL) {   // LCOV_BR_EXCL_LINE
             Py_DECREF(d);    // LCOV_EXCL_LINE
             Py_DECREF(meth); // LCOV_EXCL_LINE
-            return -1;       // LCOV_EXCL_LINE
+            return -1;
+        }
+        if (PyDict_SetItem(d, key, meth) < 0) { // LCOV_BR_EXCL_LINE
+            Py_DECREF(d);                       // LCOV_EXCL_LINE
+            Py_DECREF(meth);                    // LCOV_EXCL_LINE
+            Py_DECREF(key);                     // LCOV_EXCL_LINE
+            return -1;                          // LCOV_EXCL_LINE
         }
         Py_DECREF(meth);
+        Py_DECREF(key);
     }
 
     return PyModule_AddObject(m, "_ctests", d);
