@@ -18,8 +18,7 @@ __attribute__((__visibility__("default")))
 }
 @end
 
-@implementation
-OC_Category_GP48 (Cat)
+@implementation OC_Category_GP48 (Cat)
 - (id)gpMethod1
 {
     return @"GP48 - gpMethod1 - GP48(Cat)";
@@ -53,7 +52,8 @@ static struct PyModuleDef mod_module = {
 
 PyObject* PyInit_category_gp48(void);
 
-PyObject* __attribute__((__visibility__("default"))) PyInit_category_gp48(void)
+PyObject* __attribute__((__visibility__("default")))
+PyInit_category_gp48(void)
 {
     PyObject* m;
 
@@ -62,7 +62,9 @@ PyObject* __attribute__((__visibility__("default"))) PyInit_category_gp48(void)
         return NULL;
     }
 
-    PyObjC_ImportAPI(m);
+    if (PyObjC_ImportAPI(m) < 0) { // LCOV_BR_EXCL_LINE
+        return NULL;               // LCOV_EXCL_LINE
+    }
 
     return m;
 }
