@@ -4,8 +4,7 @@
 #import <Foundation/Foundation.h>
 #include <objc/objc.h>
 
-@interface
-NSObject ()
+@interface NSObject ()
 - (NSString*)_copyDescription;
 - (BOOL)isNSArray__;
 - (BOOL)isNSDictionary__;
@@ -20,23 +19,60 @@ NSObject ()
 - (PyObject* _Nullable)__pyobjc_PythonObject__;
 @end
 
-@interface
-NSObject (TestMethods)
+@interface NSObject (TestMethods)
 - (void)nosuchSelector;
 - (void)voidSelector;
 - (id)idSelector;
 - (id)selectorWithArg:(id)arg1 andArg:(id)arg2;
 @end
 
+@interface OC_NoPythonRepresentation : NSObject <NSCopying, NSCoding>
+/*
+ * Helper class that can be used to test error paths
+ * in creating a proxy for an ObjC object.
+ */
+{
+    bool _allowPython;
+}
+- (instancetype _Nullable)initAllowPython:(bool)allowPython;
+
+@end
+
+@interface CallHelper : NSObject {
+}
+@end
+
+@implementation CallHelper
+- (void)voidSelector
+{
+}
+@end
+
 @interface OC_ObjectInt : NSObject {
 }
 @end
 
 @implementation OC_ObjectInt
 
+- (id*)unpythonicObjects
+{
+    static id buffer[4] = {nil, nil, nil, nil};
+    if (buffer[0] == nil) {
+        buffer[0] = [[OC_NoPythonRepresentation alloc] init];
+        buffer[1] = [[OC_NoPythonRepresentation alloc] init];
+        buffer[2] = [[OC_NoPythonRepresentation alloc] init];
+    }
+    return buffer;
+}
+
 + (id)invokeSelector:(SEL)sel of:(NSObject*)object
 {
-    return ((id(*)(id, SEL))objc_msgSend)(object, sel);
+    return ((id (*)(id, SEL))objc_msgSend)(object, sel);
+}
+
++ (id)invokeSelector:(SEL)sel of:(NSObject*)object withArg:(id)arg
+{
+    return ((id (*)(id, SEL, id))objc_msgSend)(object, sel, arg);
 }
 
 + (bool)respondsToSelector:(SEL)sel classOf:(NSObject*)object
@@ -120,6 +156,24 @@ NSObject (TestMethods)
     return [first compare:second];
 }
 
++ (bool)objectEqualToNoProxy:(NSObject*)object
+{
+    NSObject* value = [[OC_NoPythonRepresentation alloc] initAllowPython:NO];
+
+    bool result = [object isEqual:value];
+    [value release];
+    return result;
+}
+
++ (NSComparisonResult)objectCompareToNoProxy:(NSObject*)object
+{
+    NSObject* value = [[OC_NoPythonRepresentation alloc] initAllowPython:NO];
+
+    NSComparisonResult result = [object compare:value];
+    [value release];
+    return result;
+}
+
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wdeprecated-declarations"
 
@@ -235,7 +289,7 @@ NSObject (TestMethods)
 
 #pragma clang diagnostic pop
 
-/* XXX: Some private methods that need to be implemented to be compabitible
+/* Some private methods that need to be implemented to be compatible
  * with NSObject
  */
 + (NSString*)copyDescriptionOf:(NSObject*)object
@@ -313,6 +367,17 @@ NSObject (TestMethods)
     return [object selectorWithArg:arg1 andArg:arg2];
 }
 
++ (id)selectorWithArgAndArgOf:(NSObject*)object
+{
+    NSObject* value = [[OC_NoPythonRepresentation alloc] initAllowPython:NO];
+
+    NSObject* result = [object selectorWithArg:value andArg:value];
+
+    [value release];
+
+    return result;
+}
+
 + (Class)invokeClassForCoderOf:(NSObject*)object
 {
     Class              result;
@@ -572,6 +637,19 @@ NSObject (TestMethods)
     [object forwardInvocation:inv];
 }
 
++ (void)invokeVoidSelectorOf:(NSObject*)object
+{
+    CallHelper*        helper = [[CallHelper alloc] init];
+    NSMethodSignature* signature =
+        [helper methodSignatureForSelector:@selector(voidSelector)];
+    [helper dealloc];
+
+    NSInvocation* inv = [NSInvocation invocationWithMethodSignature:signature];
+    inv.target        = object;
+    inv.selector      = @selector(voidSelector);
+    [object forwardInvocation:inv];
+}
+
 + (NSObject*)invokeMethodSignatureForSelector:(SEL)selector of:(NSObject*)object
 {
     NSObject*          result;
@@ -643,19 +721,38 @@ NSObject (TestMethods)
     return [[object class] accessInstanceVariablesDirectly];
 }
 
-@end
-
-@interface OC_NoPythonRepresentation : NSObject <NSCopying>
-/*
- * Helper class that can be used to test error paths
- * in creating a proxy for an ObjC object.
- */
++ (void)addObserver:(NSObject*)observer
+         forKeyPath:(NSString*)keyPath
+            options:(NSKeyValueObservingOptions)options
+            context:(void*)context
+                 on:(NSObject*)value
 {
-    bool _allowPython;
+    [value addObserver:observer forKeyPath:keyPath options:options context:context];
 }
+
++ (void)removeObserver:(NSObject*)observer
+            forKeyPath:(NSString*)keyPath
+                    on:(NSObject*)value
+{
+    [value removeObserver:observer forKeyPath:keyPath];
+}
+
 @end
 
 @implementation OC_NoPythonRepresentation
+- (void)encodeWithCoder:(NSCoder*)coder
+{
+}
+
+- (instancetype)initWithCoder:(NSCoder*)coder
+{
+    self = [super init];
+    if (!self)
+        return nil;
+
+    self->_allowPython = false;
+    return self;
+}
 - (instancetype _Nullable)init
 {
     self = [super init];
@@ -666,7 +763,7 @@ NSObject (TestMethods)
     return self;
 }
 
-- (instancetype)copyWithZone:(NSZone*)__attribute__((__unused__))zone
+- (instancetype)copyWithZone:(NSZone*)__attribute__((__unused__)) zone
 {
     return [self retain];
 }
@@ -707,34 +804,65 @@ NSObject (TestMethods)
 
 static PyMethodDef mod_methods[] = {{0, 0, 0, 0}};
 
+static int
+mod_exec_module(PyObject* m)
+{
+    if (PyObjC_ImportAPI(m) < 0) { // LCOV_BR_EXCL_LINE
+        return -1;                 // LCOV_EXCL_LINE
+    }
+
+    if (PyModule_AddObject(m, // LCOV_BR_EXCL_LINE
+                           "OC_ObjectInt", PyObjC_IdToPython([OC_ObjectInt class]))
+        < 0) {
+        return -1; // LCOV_EXCL_LINE
+    }
+
+    if (PyModule_AddObject(m, // LCOV_BR_EXCL_LINE
+                           "OC_NoPythonRepresentation",
+                           PyObjC_IdToPython([OC_NoPythonRepresentation class]))
+        < 0) {
+        return -1; // LCOV_EXCL_LINE
+    }
+    return 0;
+}
+
+static struct PyModuleDef_Slot mod_slots[] = {
+    {.slot = Py_mod_exec, .value = (void*)mod_exec_module},
+#if PY_VERSION_HEX >= 0x030c0000
+    {
+        /* This extension does not use the CPython API other than initializing
+         * the module, hence is safe with subinterpreters and per-interpreter
+         * GILs
+         */
+        .slot  = Py_mod_multiple_interpreters,
+        .value = Py_MOD_PER_INTERPRETER_GIL_SUPPORTED,
+    },
+#endif
+#if PY_VERSION_HEX >= 0x030d0000
+    {
+        .slot  = Py_mod_gil,
+        .value = Py_MOD_GIL_NOT_USED,
+    },
+#endif
+    {/* Sentinel */
+     .slot  = 0,
+     .value = 0}};
+
 static struct PyModuleDef mod_module = {
-    PyModuleDef_HEAD_INIT, "objectint", NULL, 0, mod_methods, NULL, NULL, NULL, NULL};
+    .m_base     = PyModuleDef_HEAD_INIT,
+    .m_name     = "objectint",
+    .m_doc      = NULL,
+    .m_size     = 0,
+    .m_methods  = mod_methods,
+    .m_slots    = mod_slots,
+    .m_traverse = NULL,
+    .m_clear    = NULL,
+    .m_free     = NULL,
+};
 
 PyObject* PyInit_objectint(void);
 
-PyObject* __attribute__((__visibility__("default"))) PyInit_objectint(void)
+PyObject* __attribute__((__visibility__("default"))) _Nullable PyInit_objectint(void)
 {
-    PyObject* m;
-
-    m = PyModule_Create(&mod_module);
-    if (!m) {
-        return NULL;
-    }
-
-    if (PyObjC_ImportAPI(m) < 0) {
-        return NULL;
-    }
-
-    if (PyModule_AddObject(m, "OC_ObjectInt", PyObjC_IdToPython([OC_ObjectInt class]))
-        < 0) {
-        return NULL;
-    }
-
-    if (PyModule_AddObject(m, "OC_NoPythonRepresentation",
-                           PyObjC_IdToPython([OC_NoPythonRepresentation class]))
-        < 0) {
-        return NULL;
-    }
-
-    return m;
+    return PyModuleDef_Init(&mod_module);
 }
