@@ -10,14 +10,14 @@ TODO:
 - Do the same for python 2.x
 - Do the same for sets (NSSet) and lists (NSArray)
 """
+
 import collections.abc
-import operator
 
 # Import some of the stdlib tests
 from test import mapping_tests
 
 import objc
-from PyObjCTools.TestSupport import TestCase, main
+from PyObjCTools.TestSupport import TestCase
 
 NSDictionary = objc.lookUpClass("NSDictionary")
 NSMutableDictionary = objc.lookUpClass("NSMutableDictionary")
@@ -52,7 +52,7 @@ class TestNSDictionaryInterface(TestCase):
         self.assertIn("a", k)
         self.assertIn("b", k)
 
-        self.assertIsNotInstance(k, list)
+        self.assertNotIsInstance(k, list)
 
         s = d.keys() | {"c"}
         self.assertEqual(s, {"a", "b", "c"})
@@ -89,8 +89,11 @@ class TestNSDictionaryInterface(TestCase):
         d = self.createDictionary(a=1)
 
         self.assertEqual(set(d.values()), {1})
-        self.assertRaises(TypeError, d.values, None)
-        self.assertIsNotInstance(d.values(), list)
+        with self.assertRaisesRegex(
+            TypeError, r".*\(\) takes 1 positional argument but 2 were given"
+        ):
+            d.values(None)
+        self.assertNotIsInstance(d.values(), list)
 
         self.assertEqual(
             repr(self.createDictionary(a=1).values()), "<nsdict_values([1])>"
@@ -102,7 +105,10 @@ class TestNSDictionaryInterface(TestCase):
 
         d = self.createDictionary(a=1)
         self.assertEqual(set(d.items()), {("a", 1)})
-        self.assertRaises(TypeError, d.items, None)
+        with self.assertRaisesRegex(
+            TypeError, r".*\(\) takes 1 positional argument but 2 were given"
+        ):
+            d.items(None)
         self.assertEqual(
             repr(self.createDictionary(a=1).items()), "<nsdict_items([('a', 1)])>"
         )
@@ -118,7 +124,10 @@ class TestNSDictionaryInterface(TestCase):
         self.assertIn("b", d)
         self.assertNotIn("c", d)
 
-        self.assertRaises(TypeError, d.__contains__)
+        with self.assertRaisesRegex(
+            TypeError, r".*\(\) missing 1 required positional argument: 'key'"
+        ):
+            d.__contains__()
 
     def testLen(self):
         d = self.createDictionary()
@@ -132,7 +141,10 @@ class TestNSDictionaryInterface(TestCase):
         self.assertEqual(d["a"], 1)
         self.assertEqual(d["b"], 2)
 
-        self.assertRaises(TypeError, d.__getitem__)
+        with self.assertRaisesRegex(
+            TypeError, r".*\(\) missing 1 required positional argument: 'key'"
+        ):
+            d.__getitem__()
 
     def testFromKeys(self):
         d = self.dictClass().fromkeys("abc")
@@ -148,7 +160,8 @@ class TestNSDictionaryInterface(TestCase):
             yield 1
 
         self.assertEqual(d.fromkeys(g()), {1: None})
-        self.assertRaises(TypeError, {}.fromkeys, 3)
+        with self.assertRaisesRegex(TypeError, "'int' object is not iterable"):
+            {}.fromkeys(3)
 
         d = self.dictClass()(zip(range(6), range(6)))
         self.assertEqual(dict.fromkeys(d, 0), dict(zip(range(6), [0] * 6)))
@@ -166,8 +179,14 @@ class TestNSDictionaryInterface(TestCase):
         self.assertEqual(d.get("c", 3), 3)
         self.assertEqual(d.get("a"), 1)
         self.assertEqual(d.get("a", 3), 1)
-        self.assertRaises(TypeError, d.get)
-        self.assertRaises(TypeError, d.get, None, None, None)
+        with self.assertRaisesRegex(
+            TypeError, r".*\(\) missing 1 required positional argument: 'key'"
+        ):
+            d.get()
+        with self.assertRaisesRegex(
+            TypeError, r".*\(\) takes from 2 to 3 positional arguments but 4 were given"
+        ):
+            d.get(None, None, None)
 
     def testEq(self):
         self.assertEqual(self.createDictionary(), self.createDictionary())
@@ -176,7 +195,7 @@ class TestNSDictionaryInterface(TestCase):
         class Exc(Exception):
             pass
 
-        class BadCmp(object):
+        class BadCmp:
             def __eq__(self, other):
                 raise Exc()
 
@@ -187,7 +206,7 @@ class TestNSDictionaryInterface(TestCase):
         d2 = {BadCmp(): 1}
 
         with self.assertRaises(Exc):
-            d1 == d2
+            d1 == d2  # noqa: B015
 
     def testKeysContained(self):
         self.helper_keys_contained(lambda x: x.keys())
@@ -247,19 +266,19 @@ class TestNSDictionaryInterface(TestCase):
         d2 = self.dictClass().dictionaryWithDictionary_({1: C()})
 
         with self.assertRaises(RuntimeError):
-            d1.items() == d2.items()
+            d1.items() == d2.items()  # noqa: B015
         with self.assertRaises(RuntimeError):
-            d1.items() != d2.items()
+            d1.items() != d2.items()  # noqa: B015
         with self.assertRaises(RuntimeError):
-            d1.items() <= d2.items()
+            d1.items() <= d2.items()  # noqa: B015
         with self.assertRaises(RuntimeError):
-            d1.items() >= d2.items()
+            d1.items() >= d2.items()  # noqa: B015
 
         d3 = self.dictClass().dictionaryWithDictionary_({1: C(), 2: C()})
         with self.assertRaises(RuntimeError):
-            d2.items() < d3.items()
+            d2.items() < d3.items()  # noqa: B015
         with self.assertRaises(RuntimeError):
-            d3.items() > d2.items()
+            d3.items() > d2.items()  # noqa: B015
 
         k1 = self.dictClass().dictionaryWithDictionary_({1: 1, 2: 2}).keys()
         k2 = self.dictClass().dictionaryWithDictionary_({1: 1, 2: 2, 3: 3}).keys()
@@ -347,7 +366,7 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
         self.assertIn("a", k)
         self.assertIn("b", k)
 
-        self.assertIsNotInstance(k, list)
+        self.assertNotIsInstance(k, list)
 
         self.assertNotIn("c", d)
         self.assertNotIn("c", k)
@@ -393,12 +412,15 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
         del d["b"]
         self.assertEqual(d, {"a": 4, "c": 3})
 
-        self.assertRaises(TypeError, d.__getitem__)
+        with self.assertRaisesRegex(
+            TypeError, r".*\(\) missing 1 required positional argument: 'key'"
+        ):
+            d.__getitem__()
 
         class Exc(Exception):
             pass
 
-        class BadEq(object):
+        class BadEq:
             def __eq__(self, other):
                 raise Exc()
 
@@ -407,9 +429,10 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
 
         d = self.createDictionary()
         d[BadEq()] = 42
-        self.assertRaises(KeyError, d.__getitem__, 23)
+        with self.assertRaisesRegex(KeyError, "23"):
+            d.__getitem__(23)
 
-        class BadHash(object):
+        class BadHash:
             fail = False
 
             def __hash__(self):
@@ -424,7 +447,8 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
         x.fail = True
 
         # FIXME
-        # self.assertRaises(Exc, d.__getitem__, x)
+        # with self.assertRaises(Exc):
+        #    d[x]
 
     def testClear(self):
         d = self.createDictionary(a=1, b=2, c=3)
@@ -432,7 +456,10 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
 
         self.assertEqual(d, {})
 
-        self.assertRaises(TypeError, d.clear, None)
+        with self.assertRaisesRegex(
+            TypeError, r".*\(\) takes 1 positional argument but 2 were given"
+        ):
+            d.clear(None)
 
     def testUpdate(self):
         d = self.createDictionary()
@@ -444,7 +471,10 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
         d.update()
         self.assertEqual(d, {1: 1, 2: 2, 3: 3})
 
-        self.assertRaises((TypeError, AttributeError), d.update, None)
+        with self.assertRaisesRegex(
+            (TypeError, AttributeError), "'NoneType' object is not iterable"
+        ):
+            d.update(None)
 
         class SimpleUserDict:
             def __init__(self):
@@ -469,7 +499,8 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
             def keys(self):
                 raise Exc()
 
-        self.assertRaises(Exc, d.update, FailingUserDict())
+        with self.assertRaises(Exc):
+            d.update(FailingUserDict())
 
         class FailingUserDict:
             def keys(self):
@@ -491,7 +522,8 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
             def __getitem__(self, key):
                 return key
 
-        self.assertRaises(Exc, d.update, FailingUserDict())
+        with self.assertRaises(Exc):
+            d.update(FailingUserDict())
 
         class FailingUserDict:
             def keys(self):
@@ -514,17 +546,20 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
             def __getitem__(self, key):
                 raise Exc
 
-        self.assertRaises(Exc, d.update, FailingUserDict())
+        with self.assertRaises(Exc):
+            d.update(FailingUserDict())
 
-        class badseq(object):
+        class badseq:
             def __iter__(self):
                 return self
 
             def __next__(self):
                 raise Exc()
 
-        self.assertRaises(Exc, {}.update, badseq())
-        self.assertRaises(ValueError, {}.update, [(1, 2, 3)])
+        with self.assertRaises(Exc):
+            {}.update(badseq())
+            with self.assertRaisesRegex(ValueError, "foo"):
+                {}.update([(1, 2, 3)])
 
     def setDefault(self):
         d = self.createDictionary()
@@ -535,12 +570,13 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
         self.assertEqual(d["key"][0], 3)
         d.setdefault("key", []).append(4)
         self.assertEqual(len(d["key"]), 2)
-        self.assertRaises(TypeError, d.setdefault)
+        with self.assertRaisesRegex(TypeError, "foo"):
+            d.setdefault()
 
         class Exc(Exception):
             pass
 
-        class BadHash(object):
+        class BadHash:
             fail = False
 
             def __hash__(self):
@@ -552,14 +588,15 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
         x = BadHash()
         d[x] = 42
         x.fail = True
-        self.assertRaises(Exc, d.setdefault, x, [])
+        with self.assertRaises(Exc):
+            d.setdefault(x, [])
 
     def testPopitem(self):
         for copymode in -1, +1:
             # -1: b has same structure as a
             # +1: b is a.copy()
             for log2size in range(12):
-                size = 2 ** log2size
+                size = 2**log2size
                 a = self.createDictionary()
                 b = self.createDictionary()
                 for i in range(size):
@@ -578,29 +615,37 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
                 self.assertFalse(b)
 
         d = self.createDictionary()
-        self.assertRaises(KeyError, d.popitem)
+        with self.assertRaisesRegex(
+            KeyError, "'popitem on an empty [A-Z0-9_]*NSDictionary[A-Z0-9_]*"
+        ):
+            d.popitem()
 
     def testPop(self):
         d = self.createDictionary()
         k, v = "abc", "def"
         d[k] = v
-        self.assertRaises(KeyError, d.pop, "ghi")
+        with self.assertRaisesRegex(KeyError, "^'ghi'$"):
+            d.pop("ghi")
 
         self.assertEqual(d.pop(k), v)
         self.assertEqual(len(d), 0)
 
-        self.assertRaises(KeyError, d.pop, k)
+        with self.assertRaisesRegex(KeyError, k):
+            d.pop(k)
 
         self.assertEqual(d.pop(k, v), v)
         d[k] = v
         self.assertEqual(d.pop(k, 1), v)
 
-        self.assertRaises(TypeError, d.pop)
+        with self.assertRaisesRegex(
+            TypeError, r".*\(\) missing 1 required positional argument: 'key'"
+        ):
+            d.pop()
 
         class Exc(Exception):
             pass
 
-        class BadHash(object):
+        class BadHash:
             fail = False
 
             def __hash__(self):
@@ -609,17 +654,18 @@ class TestNSMutableDictionaryInterface(TestNSDictionaryInterface):
                 else:
                     return 42
 
+        # XXX
         # x = BadHash()
         # d[x] = 42
         # x.fail = True
-        # self.assertRaises(Exc, d.pop, x)
+        # with self.assertRaises(Exc): d.pop( x)
 
 
 class DictSetTest(TestCase):
-    testclass = NSDictionary
+    dictclass = NSDictionary
 
     def testDictKeys(self):
-        d = self.testclass.dictionaryWithDictionary_({1: 10, "a": "ABC"})
+        d = self.dictclass.dictionaryWithDictionary_({1: 10, "a": "ABC"})
 
         keys = d.keys()
 
@@ -636,11 +682,11 @@ class DictSetTest(TestCase):
         self.assertNotIn("Z", keys)
         self.assertEqual(d.keys(), d.keys())
 
-        e = self.testclass.dictionaryWithDictionary_({1: 11, "a": "def"})
+        e = self.dictclass.dictionaryWithDictionary_({1: 11, "a": "def"})
         self.assertEqual(d.keys(), e.keys())
 
     def testDictItems(self):
-        d = self.testclass.dictionaryWithDictionary_({1: 10, "a": "ABC"})
+        d = self.dictclass.dictionaryWithDictionary_({1: 10, "a": "ABC"})
         items = d.items()
 
         self.assertEqual(len(items), 2)
@@ -664,18 +710,18 @@ class DictSetTest(TestCase):
 
 
 class DictSetTest(DictSetTest):
-    testclass = NSMutableDictionary
+    dictclass = NSMutableDictionary
 
     def testDictKeysMutable(self):
-        d = self.testclass.dictionaryWithDictionary_({1: 10, "a": "ABC"})
-        e = self.testclass.dictionaryWithDictionary_({1: 11, "a": "def"})
+        d = self.dictclass.dictionaryWithDictionary_({1: 10, "a": "ABC"})
+        e = self.dictclass.dictionaryWithDictionary_({1: 11, "a": "def"})
         self.assertEqual(d.keys(), e.keys())
 
         del e["a"]
         self.assertNotEqual(d.keys(), e.keys())
 
     def testDictItemsMutable(self):
-        d = self.testclass.dictionaryWithDictionary_({1: 10, "a": "ABC"})
+        d = self.dictclass.dictionaryWithDictionary_({1: 10, "a": "ABC"})
 
         e = d.copy()
         self.assertEqual(d.items(), e.items())
@@ -683,13 +729,13 @@ class DictSetTest(DictSetTest):
         self.assertNotEqual(d.items(), e.items())
 
     def testDictMixedKeysItems(self):
-        d = self.testclass.dictionaryWithDictionary_({(1, 1): 11, (2, 2): 22})
-        e = self.testclass.dictionaryWithDictionary_({1: 1, 2: 2})
+        d = self.dictclass.dictionaryWithDictionary_({(1, 1): 11, (2, 2): 22})
+        e = self.dictclass.dictionaryWithDictionary_({1: 1, 2: 2})
         self.assertEqual(d.keys(), e.items())
         self.assertNotEqual(d.items(), e.keys())
 
     def testDictValues(self):
-        d = self.testclass.dictionaryWithDictionary_({1: 10, "a": "ABC"})
+        d = self.dictclass.dictionaryWithDictionary_({1: 10, "a": "ABC"})
         values = d.values()
         self.assertEqual(set(values), {10, "ABC"})
         self.assertEqual(len(values), 2)
@@ -703,7 +749,10 @@ class TestDictUpdates(TestCase):
     def do_test(self, dictType):
         d = dictType()
         d["a"] = 42
-        self.assertRaises(TypeError, d.update, {}, {})
+        with self.assertRaisesRegex(
+            TypeError, "update expected at most 1 argument(s)?, got 2"
+        ):
+            d.update({}, {})
 
         d.update({"b": 9})
         self.assertEqual(dict(d), {"a": 42, "b": 9})
@@ -747,17 +796,63 @@ class TestABC(TestCase):
 
 class TestPyObjCDict(TestCase):
     def test_comparison(self):
-        self.assertRaises(TypeError, operator.lt, {}, {})
-        self.assertRaises(TypeError, operator.le, {}, {})
-        self.assertRaises(TypeError, operator.gt, {}, {})
-        self.assertRaises(TypeError, operator.ge, {}, {})
+        with self.assertRaisesRegex(
+            TypeError, "'<' not supported between instances of 'dict' and 'dict'"
+        ):
+            {} < {}  # noqa: B015
+        with self.assertRaisesRegex(
+            TypeError, "'<=' not supported between instances of 'dict' and 'dict'"
+        ):
+            {} <= {}  # noqa: B015
+        with self.assertRaisesRegex(
+            TypeError, "'>' not supported between instances of 'dict' and 'dict'"
+        ):
+            {} > {}  # noqa: B015
+        with self.assertRaisesRegex(
+            TypeError, "'>=' not supported between instances of 'dict' and 'dict'"
+        ):
+            {} >= {}  # noqa: B015
 
-        self.assertRaises(TypeError, operator.lt, NSDictionary(), {})
-        self.assertRaises(TypeError, operator.le, NSDictionary(), {})
-        self.assertRaises(TypeError, operator.gt, NSDictionary(), {})
-        self.assertRaises(TypeError, operator.lt, {}, NSDictionary())
-        self.assertRaises(TypeError, operator.le, {}, NSDictionary())
-        self.assertRaises(TypeError, operator.gt, {}, NSDictionary())
+        with self.assertRaisesRegex(
+            TypeError,
+            "'<' not supported between instances of '[^']*NSDictionary[^']*' and 'dict'",
+        ):
+            NSDictionary() < {}  # noqa: B015
+        with self.assertRaisesRegex(
+            TypeError,
+            "'<=' not supported between instances of '[^']*NSDictionary[^']*' and 'dict'",
+        ):
+            NSDictionary() <= {}  # noqa: B015
+        with self.assertRaisesRegex(
+            TypeError,
+            "'>' not supported between instances of '[^']*NSDictionary[^']*' and 'dict'",
+        ):
+            NSDictionary() > {}  # noqa: B015
+        with self.assertRaisesRegex(
+            TypeError,
+            "'>=' not supported between instances of '[^']*NSDictionary[^']*' and 'dict'",
+        ):
+            NSDictionary() >= {}  # noqa: B015
+        with self.assertRaisesRegex(
+            TypeError,
+            "'<' not supported between instances of 'dict' and '[^']*NSDictionary[^']*'",
+        ):
+            {} < NSDictionary()  # noqa: B015
+        with self.assertRaisesRegex(
+            TypeError,
+            "'<=' not supported between instances of 'dict' and '[^']*NSDictionary[^']*'",
+        ):
+            {} <= NSDictionary()  # noqa: B015
+        with self.assertRaisesRegex(
+            TypeError,
+            "'>' not supported between instances of 'dict' and '[^']*NSDictionary[^']*'",
+        ):
+            {} > NSDictionary()  # noqa: B015
+        with self.assertRaisesRegex(
+            TypeError,
+            "'>=' not supported between instances of 'dict' and '[^']*NSDictionary[^']*'",
+        ):
+            {} >= NSDictionary()  # noqa: B015
 
         self.assertFalse(NSMutableDictionary() == [])
         self.assertFalse(NSDictionary() == [])
@@ -790,7 +885,10 @@ class TestPyObjCDict(TestCase):
             self.assertEqual(v[1], -1)
             self.assertEqual(v[2], 9)
 
-            self.assertRaises(TypeError, dict_type, (1, 2), (3, 4))
+            with self.assertRaisesRegex(
+                TypeError, "dict expected at most 1 arguments, got 2"
+            ):
+                dict_type((1, 2), (3, 4))
 
             v = dict_type(a=3, b=4)
             self.assertEqual(len(v), 2)
@@ -820,32 +918,70 @@ class TestPyObjCDict(TestCase):
         v = oc.keys() | {"a", "f"}
         self.assertEqual(v, {"a", "b", "c", "d", "e", "f"})
         self.assertIsInstance(v, set)
-        self.assertRaises(TypeError, operator.or_, oc.keys(), ("a", "f"))
-        self.assertRaises(TypeError, operator.or_, ("a", "f"), oc.keys())
+        with self.assertRaisesRegex(
+            TypeError, r"unsupported operand type\(s\) for |: 'nsdict_keys' and 'tuple'"
+        ):
+            oc.keys() | ("a", "f")
+        with self.assertRaisesRegex(
+            TypeError, r"unsupported operand type\(s\) for |: 'tuple' and 'nsdict_keys'"
+        ):
+            ("a", "f") | oc.keys()
 
         v = oc.keys() & {"a", "f"}
         self.assertEqual(v, {"a"})
         self.assertIsInstance(v, set)
-        self.assertRaises(TypeError, operator.and_, oc.keys(), ("a", "f"))
-        self.assertRaises(TypeError, operator.and_, ("a", "f"), oc.keys())
+        with self.assertRaisesRegex(
+            TypeError, r"unsupported operand type\(s\) for &: 'nsdict_keys' and 'tuple'"
+        ):
+            oc.keys() & ("a", "f")
+        with self.assertRaisesRegex(
+            TypeError, r"unsupported operand type\(s\) for &: 'tuple' and 'nsdict_keys'"
+        ):
+            ("a", "f") & oc.keys()
 
         v = oc.keys() ^ {"a", "f"}
         self.assertEqual(v, {"b", "c", "d", "e", "f"})
         self.assertIsInstance(v, set)
-        self.assertRaises(TypeError, operator.xor, oc.keys(), ("a", "f"))
-        self.assertRaises(TypeError, operator.xor, ("a", "f"), oc.keys())
+        with self.assertRaisesRegex(
+            TypeError,
+            r"unsupported operand type\(s\) for \^: 'nsdict_keys' and 'tuple'",
+        ):
+            oc.keys() ^ ("a", "f")
+        with self.assertRaisesRegex(
+            TypeError,
+            r"unsupported operand type\(s\) for \^: 'tuple' and 'nsdict_keys'",
+        ):
+            ("a", "f") ^ oc.keys()
 
         v = oc.keys() - {"a", "f"}
         self.assertEqual(v, {"b", "c", "d", "e"})
         self.assertIsInstance(v, set)
-        self.assertRaises(TypeError, operator.sub, oc.keys(), ("a", "f"))
-        self.assertRaises(TypeError, operator.sub, ("a", "f"), oc.keys())
+        with self.assertRaisesRegex(
+            TypeError, r"unsupported operand type\(s\) for -: 'nsdict_keys' and 'tuple'"
+        ):
+            oc.keys() - ("a", "f")
+        with self.assertRaisesRegex(
+            TypeError, r"unsupported operand type\(s\) for -: 'tuple' and 'nsdict_keys'"
+        ):
+            ("a", "f") - oc.keys()
 
-        self.assertRaises(TypeError, operator.lt, oc.keys(), ("a", "f"))
-        self.assertRaises(TypeError, operator.le, oc.keys(), ("a", "f"))
-        self.assertRaises(TypeError, operator.gt, oc.keys(), ("a", "f"))
-        self.assertRaises(TypeError, operator.ge, oc.keys(), ("a", "f"))
-
-
-if __name__ == "__main__":
-    main()
+        with self.assertRaisesRegex(
+            TypeError,
+            r"'<' not supported between instances of 'nsdict_keys' and 'tuple'",
+        ):
+            oc.keys() < ("a", "f")  # noqa: B015
+        with self.assertRaisesRegex(
+            TypeError,
+            r"'<=' not supported between instances of 'nsdict_keys' and 'tuple'",
+        ):
+            oc.keys() <= ("a", "f")  # noqa: B015
+        with self.assertRaisesRegex(
+            TypeError,
+            r"'>' not supported between instances of 'nsdict_keys' and 'tuple'",
+        ):
+            oc.keys() > ("a", "f")  # noqa: B015
+        with self.assertRaisesRegex(
+            TypeError,
+            r"'>=' not supported between instances of 'nsdict_keys' and 'tuple'",
+        ):
+            oc.keys() >= ("a", "f")  # noqa: B015
