@@ -4,8 +4,10 @@ Tests for the Key-Value Coding support in OC_PythonObject
 NOTE: Testcases here should be synchronized with the Key-Value Coding tests
 in PyObjCTools.test.test_keyvalue and Foundation.test.test_keyvalue.
 """
+
 import os
 import sys
+from objc import super  # noqa: A004
 
 import objc
 from PyObjCTest.fnd import (
@@ -41,15 +43,17 @@ from PyObjCTest.keyvaluehelper import (
     PyObjCTest_KVPathClass,
     PyObjCTest_KVBaseClass,
 )
+from .objectint import OC_ObjectInt
+from .test_metadata import NoObjCClass
 
 # Native code is needed to access the python class from Objective-C, otherwise
 # the Key-Value support cannot be tested.
 from PyObjCTest.testbndl import PyObjC_TestClass3 as STUB
 from PyObjCTest.testbndl import PyObjCTest_KeyValueObserver
-from PyObjCTools.TestSupport import TestCase, main, min_os_level
+from PyObjCTools.TestSupport import TestCase, min_os_level
 
 
-class KeyValueClass2(object):
+class KeyValueClass2:
     def __init__(self):
         self.key3 = 3
         self._key4 = "4"
@@ -83,7 +87,7 @@ class KeyValueClass2(object):
         self.key5 = value * 5
 
 
-class KeyValueClass3(object):
+class KeyValueClass3:
     __slots__ = ("foo",)
 
     def __init__(self):
@@ -107,9 +111,8 @@ class PyKeyValueCoding(TestCase):
         # Private instance variables ('anObject.__value') are not accessible using
         # key-value coding.
         o = KeyValueClass2()
-        self.assertRaises(
-            KeyError, STUB.keyValue_forObject_key_, DO_VALUEFORKEY, o, "private"
-        )
+        with self.assertRaisesRegex(KeyError, "private"):
+            STUB.keyValue_forObject_key_(DO_VALUEFORKEY, o, "private")
 
     def testValueForKey(self):
         o = KeyValueClass2()
@@ -126,9 +129,8 @@ class PyKeyValueCoding(TestCase):
             STUB.keyValue_forObject_key_(DO_VALUEFORKEY, o, "pythonConvention"), "GOOD"
         )
 
-        self.assertRaises(
-            KeyError, STUB.keyValue_forObject_key_, DO_VALUEFORKEY, o, "nokey"
-        )
+        with self.assertRaisesRegex(KeyError, "nokey"):
+            STUB.keyValue_forObject_key_(DO_VALUEFORKEY, o, "nokey")
 
     def testValueForKey2(self):
         o = KeyValueClass3()
@@ -164,9 +166,8 @@ class PyKeyValueCoding(TestCase):
             o.multiple,
         )
 
-        self.assertRaises(
-            KeyError, STUB.keyValue_forObject_key_, DO_STOREDVALUEFORKEY, o, "nokey"
-        )
+        with self.assertRaisesRegex(KeyError, "nokey"):
+            STUB.keyValue_forObject_key_(DO_STOREDVALUEFORKEY, o, "nokey")
 
     def testStoredValueForKey2(self):
         o = KeyValueClass3()
@@ -205,13 +206,12 @@ class PyKeyValueCoding(TestCase):
             o.multiple.level2.level3.keyB,
         )
 
-        self.assertRaises(
-            KeyError,
-            STUB.keyValue_forObject_key_,
-            DO_VALUEFORKEYPATH,
-            o,
-            "multiple.level2.nokey",
-        )
+        with self.assertRaisesRegex(KeyError, "nokey"):
+            STUB.keyValue_forObject_key_(
+                DO_VALUEFORKEYPATH,
+                o,
+                "multiple.level2.nokey",
+            )
 
     def testValuesForKeys(self):
         o = KeyValueClass2()
@@ -223,13 +223,12 @@ class PyKeyValueCoding(TestCase):
             {"key1": 1, "key2": 2, "key3": 3, "key4": "4"},
         )
 
-        self.assertRaises(
-            KeyError,
-            STUB.keyValue_forObject_key_,
-            DO_VALUESFORKEYS,
-            o,
-            ["key1", "key2", "nokey", "key3"],
-        )
+        with self.assertRaisesRegex(KeyError, "nokey"):
+            STUB.keyValue_forObject_key_(
+                DO_VALUESFORKEYS,
+                o,
+                ["key1", "key2", "nokey", "key3"],
+            )
 
     def testTakeValueForKey(self):
         o = KeyValueClass2()
@@ -258,14 +257,13 @@ class PyKeyValueCoding(TestCase):
         STUB.setKeyValue_forObject_key_value_(DO_TAKEVALUE_FORKEY, o, "foo", "FOO")
         self.assertEqual(o.foo, "FOO")
 
-        self.assertRaises(
-            KeyError,
-            STUB.setKeyValue_forObject_key_value_,
-            DO_TAKEVALUE_FORKEY,
-            o,
-            "key9",
-            "IX",
-        )
+        with self.assertRaisesRegex(KeyError, "key9"):
+            STUB.setKeyValue_forObject_key_value_(
+                DO_TAKEVALUE_FORKEY,
+                o,
+                "key9",
+                "IX",
+            )
 
     def testTakeStoredValueForKey(self):
         o = KeyValueClass2()
@@ -302,22 +300,20 @@ class PyKeyValueCoding(TestCase):
         )
         self.assertEqual(o.foo, "FOO")
 
-        self.assertRaises(
-            KeyError,
-            STUB.setKeyValue_forObject_key_value_,
-            DO_TAKESTOREDVALUE_FORKEY,
-            o,
-            "key9",
-            "IX",
-        )
-        self.assertRaises(
-            KeyError,
-            STUB.setKeyValue_forObject_key_value_,
-            DO_TAKESTOREDVALUE_FORKEY,
-            o,
-            "roprop",
-            "IX",
-        )
+        with self.assertRaisesRegex(KeyError, "Key key9 does not exist'"):
+            STUB.setKeyValue_forObject_key_value_(
+                DO_TAKESTOREDVALUE_FORKEY,
+                o,
+                "key9",
+                "IX",
+            )
+        with self.assertRaisesRegex(KeyError, "Key roprop does not exist'"):
+            STUB.setKeyValue_forObject_key_value_(
+                DO_TAKESTOREDVALUE_FORKEY,
+                o,
+                "roprop",
+                "IX",
+            )
 
     def testTakeValuesFromDictionary(self):
         o = KeyValueClass2()
@@ -349,22 +345,20 @@ class PyKeyValueCoding(TestCase):
         )
         self.assertEqual(o.foo, "FOO")
 
-        self.assertRaises(
-            KeyError,
-            STUB.setKeyValue_forObject_key_value_,
-            DO_TAKEVALUESFROMDICT,
-            o,
-            None,
-            {"key9": "IX"},
-        )
-        self.assertRaises(
-            KeyError,
-            STUB.setKeyValue_forObject_key_value_,
-            DO_TAKEVALUESFROMDICT,
-            o,
-            None,
-            {"roprop": "IX"},
-        )
+        with self.assertRaisesRegex(KeyError, "Key key9 does not exist"):
+            STUB.setKeyValue_forObject_key_value_(
+                DO_TAKEVALUESFROMDICT,
+                o,
+                None,
+                {"key9": "IX"},
+            )
+        with self.assertRaisesRegex(KeyError, "Key roprop does not exist"):
+            STUB.setKeyValue_forObject_key_value_(
+                DO_TAKEVALUESFROMDICT,
+                o,
+                None,
+                {"roprop": "IX"},
+            )
 
     def testTakeValueForKeyPath(self):
         o = KeyValueClass2()
@@ -398,20 +392,24 @@ class TestAccMethod(TestCase):
     def testStr(self):
         # Strings are automaticly converted to NSStrings, and those don't have
         # a capitalize key.
-        self.assertRaises(
+        with self.assertRaisesRegex(
             KeyError,
-            STUB.keyValue_forObject_key_,
-            DO_VALUEFORKEY,
-            "hello",
-            "capitalize",
-        )
-        self.assertRaises(
+            r"NSUnknownKeyException - \[.*\]: this class is not key value coding-compliant for the key capitalize.",
+        ):
+            STUB.keyValue_forObject_key_(
+                DO_VALUEFORKEY,
+                "hello",
+                "capitalize",
+            )
+        with self.assertRaisesRegex(
             KeyError,
-            STUB.keyValue_forObject_key_,
-            DO_VALUEFORKEY,
-            "hello",
-            "capitalize",
-        )
+            r"NSUnknownKeyException - \[.*\]: this class is not key value coding-compliant for the key capitalize.",
+        ):
+            STUB.keyValue_forObject_key_(
+                DO_VALUEFORKEY,
+                "hello",
+                "capitalize",
+            )
 
 
 class AbstractKVCodingTest:
@@ -547,7 +545,6 @@ class TestPythonSubOverObjC(AbstractKVCodingTest, TestCase):
 
 
 if sys.platform == "darwin" and os.uname()[2] >= "7.0.0":
-
     # macOS 10.3 and later use 'setValue:forKey: u' instead of
     # 'takeValue:forKey: u', test these as wel.
 
@@ -600,14 +597,13 @@ if sys.platform == "darwin" and os.uname()[2] >= "7.0.0":
             STUB.setKeyValue_forObject_key_value_(DO_SETVALUE_FORKEY, o, "foo", "FOO")
             self.assertEqual(o.foo, "FOO")
 
-            self.assertRaises(
-                KeyError,
-                STUB.setKeyValue_forObject_key_value_,
-                DO_SETVALUE_FORKEY,
-                o,
-                "key9",
-                "IX",
-            )
+            with self.assertRaisesRegex(KeyError, "foo"):
+                STUB.setKeyValue_forObject_key_value_(
+                    DO_SETVALUE_FORKEY,
+                    o,
+                    "key9",
+                    "IX",
+                )
 
         def testSetValuesForKeysFromDictionary(self):
             o = KeyValueClass2()
@@ -639,22 +635,20 @@ if sys.platform == "darwin" and os.uname()[2] >= "7.0.0":
             )
             self.assertEqual(o.foo, "FOO")
 
-            self.assertRaises(
-                KeyError,
-                STUB.setKeyValue_forObject_key_value_,
-                DO_SETVALUESFORKEYSFROMDICT,
-                o,
-                None,
-                {"key9": "IX"},
-            )
-            self.assertRaises(
-                KeyError,
-                STUB.setKeyValue_forObject_key_value_,
-                DO_SETVALUESFORKEYSFROMDICT,
-                o,
-                None,
-                {"roprop": "IX"},
-            )
+            with self.assertRaisesRegex(KeyError, "foo"):
+                STUB.setKeyValue_forObject_key_value_(
+                    DO_SETVALUESFORKEYSFROMDICT,
+                    o,
+                    None,
+                    {"key9": "IX"},
+                )
+            with self.assertRaisesRegex(KeyError, "foo"):
+                STUB.setKeyValue_forObject_key_value_(
+                    DO_SETVALUESFORKEYSFROMDICT,
+                    o,
+                    None,
+                    {"roprop": "IX"},
+                )
 
         def testSetValueForKeyPath(self):
             o = KeyValueClass2()
@@ -934,5 +928,146 @@ if PyObjCTest_KeyValueObserver is not None:
                 o.removeObserver_forKeyPath_(observer, "observationInfo")
 
 
-if __name__ == "__main__":
-    main()
+class Helper:
+    pass
+
+
+class TestWithoutHelper(TestCase):
+    def test_get_keypath(self):
+        orig = objc.options._getKeyPath
+        try:
+            objc.options._getKeyPath = None
+
+            value = Helper()
+            value.key = 42
+
+            a = NSArray.arrayWithArray_([value])
+            with self.assertRaisesRegex(
+                ValueError, "helper function for getKeyPath not set"
+            ):
+                a.valueForKeyPath_("@unionOfObjects.key")
+
+        finally:
+            objc.options._getKeyPath = orig
+
+    def test_get_key(self):
+        orig = objc.options._getKey
+        try:
+            objc.options._getKey = None
+
+            value = Helper()
+            value.key = 42
+
+            a = NSArray.arrayWithArray_([value])
+            with self.assertRaisesRegex(
+                ValueError, "helper function for getKey not set"
+            ):
+                a.makeObjectsPerformSelector_withObject_(b"valueForKey:", "key")
+
+        finally:
+            objc.options._getKey = orig
+
+    def test_set_key(self):
+        orig = objc.options._setKey
+        try:
+            objc.options._setKey = None
+
+            value = Helper()
+            value.key = 42
+
+            with self.assertRaisesRegex(
+                ValueError, "helper function for setKey not set"
+            ):
+                OC_ObjectInt.setValue_forKey_of_("value", "key", value)
+
+            self.assertEqual(value.key, 42)
+
+        finally:
+            objc.options._setKey = orig
+
+    def test_set_keypath(self):
+        orig = objc.options._setKeyPath
+        try:
+            objc.options._setKeyPath = None
+
+            value = Helper()
+            value.key = 42
+
+            with self.assertRaisesRegex(
+                ValueError, "helper function for setKeyPath not set"
+            ):
+                OC_ObjectInt.setValue_forKeyPath_of_("value", "key.path", value)
+
+            objc.options._setKeyPath = 42
+            with self.assertRaisesRegex(TypeError, "not callable"):
+                OC_ObjectInt.setValue_forKeyPath_of_("value", "key.path", value)
+
+            self.assertEqual(value.key, 42)
+
+        finally:
+            objc.options._setKeyPath = orig
+
+
+class OC_KeyValueTestHelperWithSlots(NSObject):
+    __slots__ = ("a", "_b")
+
+    def init(self):
+        self = super().init()
+        if self is None:
+            return None
+
+        return self
+
+
+class TestKeyValueObjC(TestCase):
+    def test_setting(self):
+        o = OC_KeyValueTestHelperWithSlots.alloc().init()
+
+        self.assertNotHasAttr(o, "a")
+        self.assertNotHasAttr(o, "b")
+        self.assertNotHasAttr(o, "_a")
+        self.assertNotHasAttr(o, "_b")
+
+        with self.assertRaisesRegex(
+            KeyError, "this class is not key value coding-compliant for the key b."
+        ):
+            OC_ObjectInt.setValue_forKey_of_("BB", "b", o)
+
+        o._b = "bb"
+        OC_ObjectInt.setValue_forKey_of_("AA", "a", o)
+        OC_ObjectInt.setValue_forKey_of_("BB", "b", o)
+
+        self.assertNotHasAttr(o, "_a")
+        self.assertNotHasAttr(o, "b")
+        self.assertEqual(o.a, "AA")
+        self.assertEqual(o._b, "BB")
+
+    def test_getting(self):
+        o = OC_KeyValueTestHelperWithSlots.alloc().init()
+        o.a = "Aa"
+        o._b = "Bb"
+
+        v = OC_ObjectInt.valueForKey_of_("a", o)
+        self.assertEqual(v, "Aa")
+        v = OC_ObjectInt.valueForKey_of_("b", o)
+        self.assertEqual(v, "Bb")
+
+        # XXX: This is not ideal, but protects against returning a bound method
+        #      (not an attribute whose's value happens to be a bound method)
+        o._b = o.description
+        with self.assertRaisesRegex(
+            KeyError, "this class is not key value coding-compliant for the key b."
+        ):
+            OC_ObjectInt.valueForKey_of_("b", o)
+
+        o.a = NoObjCClass()
+        with self.assertRaisesRegex(
+            KeyError, "this class is not key value coding-compliant for the key a."
+        ):
+            OC_ObjectInt.valueForKey_of_("a", o)
+
+        del o._b
+        with self.assertRaisesRegex(
+            KeyError, "this class is not key value coding-compliant for the key b."
+        ):
+            OC_ObjectInt.valueForKey_of_("b", o)
