@@ -11,14 +11,19 @@ TODO:
 - The python->C interface (that is the contents of the metadata object) is
   likely to change when the bridge is feature-complete.
 - Probably need special-casing for arrays (numarray and array.array)!
+
+XXX: python-to-python calls often don't pass through PyObjC's machinery,
+     resulting in slightly different behaviour. Fixing this likely breaks
+     user code.
 """
+
 import objc
 
 # To ensure we have the right metadata
 import PyObjCTest.test_metadata  # noqa: F401
 from PyObjCTest.metadata import OC_MetaDataTest
 from PyObjCTest.test_metadata_py import Py_MetaDataTest_AllArgs
-from PyObjCTools.TestSupport import TestCase, main
+from PyObjCTools.TestSupport import TestCase
 
 if 0:
     from PyObjCTest.test_metadata_py2 import Py_MetaDataTest_OutputOptional
@@ -28,17 +33,11 @@ class TestArraysOut_AllArgs(TestCase):
     def testFixedSize(self):
         o = Py_MetaDataTest_AllArgs.new()
 
-        v = o.fill4Tuple_()
-        self.assertEqual(list(v), list(range(9, 13)))
-
         v = o.fill4Tuple_(None)
         self.assertEqual(list(v), list(range(9, 13)))
 
-        self.assertRaises(ValueError, OC_MetaDataTest.fill4Tuple_on_, objc.NULL, o)
-
-        n, v = o.nullfill4Tuple_()
-        self.assertEqual(n, 1)
-        self.assertEqual(list(v), list(range(1, 5)))
+        with self.assertRaisesRegex(ValueError, "argument 0 isn't allowed to be NULL"):
+            OC_MetaDataTest.fill4Tuple_on_(objc.NULL, o)
 
         n, v = o.nullfill4Tuple_(None)
         self.assertEqual(n, 1)
@@ -46,6 +45,7 @@ class TestArraysOut_AllArgs(TestCase):
 
         n, v = o.nullfill4Tuple_(objc.NULL)
         self.assertEqual(n, 2)
+        # FIXME:
         # self.assertIs(v, objc.NULL )
 
     def testNullTerminated(self):
@@ -68,23 +68,17 @@ class TestArraysOut_AllArgs(TestCase):
     def testWithCount(self):
         o = Py_MetaDataTest_AllArgs.new()
 
-        v = o.fillArray_count_(3)
-        self.assertEqual(list(v), [10, 11, 12])
-
         v = o.fillArray_count_(None, 3)
         self.assertEqual(list(v), [10, 11, 12])
 
-        v = o.fillArray_count_(5)
+        v = o.fillArray_count_(None, 5)
         self.assertEqual(list(v), [10, 11, 12, 13, 14])
 
-        v = o.fillArray_count_(0)
+        v = o.fillArray_count_(None, 0)
         self.assertEqual(list(v), [])
 
         # self.assertRaises(ValueError, o.fillArray_count_, objc.NULL, 0)
 
-        n, v = o.nullfillArray_count_(3)
-        self.assertEqual(n, 2)
-        self.assertEqual(list(v), [30, 31, 32])
         n, v = o.nullfillArray_count_(None, 3)
         self.assertEqual(n, 2)
         self.assertEqual(list(v), [30, 31, 32])
@@ -96,11 +90,11 @@ class TestArraysOut_AllArgs(TestCase):
     def testWithCountInResult(self):
         o = Py_MetaDataTest_AllArgs.new()
 
-        c, v = o.fillArray_uptoCount_(20)
+        c, v = o.fillArray_uptoCount_(None, 20)
         self.assertEqual(c, 10)
         self.assertEqual(list(v), [i + 10 for i in range(10)])
 
-        c, v = o.maybeFillArray_()
+        c, v = o.maybeFillArray_(None)
         self.assertEqual(c, 2)
         self.assertEqual(list(v), [0, 1])
 
@@ -117,7 +111,10 @@ if 0:
             v = o.fill4Tuple_(None)
             self.assertEqual(list(v), list(range(9, 13)))
 
-            self.assertRaises(ValueError, OC_MetaDataTest.fill4Tuple_on_, objc.NULL, o)
+            with self.assertRaisesRegex(
+                ValueError, "argument 0 isn't allowed to be NULL"
+            ):
+                OC_MetaDataTest.fill4Tuple_on_(objc.NULL, o)
 
             n, v = o.nullfill4Tuple_()
             self.assertEqual(n, 1)
@@ -323,8 +320,10 @@ class TestArraysIn_AllArgs(TestCase):
         (v,) = OC_MetaDataTest.makeStringArray_on_((), o)
         self.assertEqual(len(v), 0)
 
-        self.assertRaises(ValueError, OC_MetaDataTest.makeStringArray_on_, [1, 2], o)
-        self.assertRaises(ValueError, OC_MetaDataTest.makeStringArray_on_, objc.NULL, o)
+        with self.assertRaisesRegex(ValueError, "depythonifying 'charptr', got 'int'"):
+            OC_MetaDataTest.makeStringArray_on_([1, 2], o)
+        with self.assertRaisesRegex(ValueError, "argument 0 isn't allowed to be NULL"):
+            OC_MetaDataTest.makeStringArray_on_(objc.NULL, o)
 
         (v,) = OC_MetaDataTest.nullStringArray_on_(objc.NULL, o)
         self.assertEqual(v, objc.NULL)
@@ -414,21 +413,21 @@ class TestByReference_AllArgs(TestCase):
         o = Py_MetaDataTest_AllArgs.new()
 
         r = o.sumX_andY_(1, 2)
-        self.assertEqual(r, 1 ** 2 + 2 ** 2)
+        self.assertEqual(r, 1**2 + 2**2)
 
         r = o.sumX_andY_(2535, 5325)
-        self.assertEqual(r, 2535 ** 2 + 5325 ** 2)
+        self.assertEqual(r, 2535**2 + 5325**2)
 
         # self.assertRaises(ValueError, o.sumX_andY_, 42, objc.NULL)
 
     def testOutput(self):
         o = Py_MetaDataTest_AllArgs.new()
 
-        div, rem = o.divBy5_remainder_(55)
+        div, rem = o.divBy5_remainder_(55, None)
         self.assertEqual(div, 55 / 7)
         self.assertEqual(rem, 55 % 7)
 
-        div, rem = o.divBy5_remainder_(13)
+        div, rem = o.divBy5_remainder_(13, None)
         self.assertEqual(div, 13 / 7)
         self.assertEqual(rem, 13 % 7)
 
@@ -451,7 +450,7 @@ class TestByReference_AllArgs(TestCase):
             return int(value, 0)
 
         # All arguments present
-        r, y, z = o.input_output_inputAndOutput_(1, 2)
+        r, y, z = o.input_output_inputAndOutput_(1, None, 2)  # XXX
         self.assertEqual(len(r), 3)
         self.assertEqual(len(list(filter(lambda x: x is not objc.NULL, r))), 3)
         self.assertEqual(y, 9)
@@ -464,12 +463,6 @@ class TestByReference_AllArgs(TestCase):
         self.assertEqual(z, 10)
 
         # Argument 1 is NULL
-        r, y, z = o.input_output_inputAndOutput_(objc.NULL, 2)
-        self.assertEqual(len(r), 3)
-        self.assertEqual(len(list(filter(lambda x: x is not objc.NULL, r))), 2)
-        self.assertEqual(y, 11)
-        self.assertEqual(z, 12)
-
         r, y, z = o.input_output_inputAndOutput_(objc.NULL, None, 2)
         self.assertEqual(len(r), 3)
         self.assertEqual(len(list(filter(lambda x: x is not objc.NULL, r))), 2)
@@ -484,18 +477,8 @@ class TestByReference_AllArgs(TestCase):
         self.assertEqual(z, 14)
 
         # Argument 3 is NULL
-        r, y, z = o.input_output_inputAndOutput_(1, objc.NULL)
-        self.assertEqual(len(r), 3)
-        self.assertEqual(len(list(filter(lambda x: x is not objc.NULL, r))), 2)
-        self.assertEqual(y, 15)
-        self.assertEqual(z, 16)  # objc.NULL ...
-
         r, y, z = o.input_output_inputAndOutput_(1, None, objc.NULL)
         self.assertEqual(len(r), 3)
         self.assertEqual(len(list(filter(lambda x: x is not objc.NULL, r))), 2)
         self.assertEqual(y, 15)
         self.assertEqual(z, 16)  # , objc.NULL)
-
-
-if __name__ == "__main__":
-    main()
