@@ -1,5 +1,10 @@
 import AVFoundation
-from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure
+from PyObjCTools.TestSupport import (
+    TestCase,
+    min_os_level,
+    min_sdk_level,
+    expectedFailure,
+)
 
 
 class TestAVCaptureSession(TestCase):
@@ -55,6 +60,14 @@ class TestAVCaptureSession(TestCase):
             AVFoundation.AVCaptureSessionInterruptionEndedNotification, str
         )
 
+    @min_sdk_level("15.2")
+    def test_protocools(self):
+        self.assertProtocolExists("AVCaptureSessionControlsDelegate")
+
+    @min_sdk_level("26.0")
+    def test_protocools26_0(self):
+        self.assertProtocolExists("AVCaptureSessionDeferredStartDelegate")
+
     @min_os_level("10.7")
     def testMethods(self):
         self.assertResultIsBOOL(AVFoundation.AVCaptureSession.canSetSessionPreset_)
@@ -158,3 +171,17 @@ class TestAVCaptureSession(TestCase):
         self.assertResultIsBOOL(
             AVFoundation.AVCaptureConnection.isVideoRotationAngleSupported_
         )
+
+    @min_os_level("15.0")
+    def testMethods15_0(self):
+        self.assertResultIsBOOL(AVFoundation.AVCaptureSession.supportsControls)
+        self.assertResultIsBOOL(AVFoundation.AVCaptureSession.canAddControl_)
+
+    @min_os_level("26.0")
+    def testMethods26_0(self):
+        self.assertResultIsBOOL(
+            AVFoundation.AVCaptureSession.automaticallyRunsDeferredStart
+        )
+        self.assertArgIsBOOL(
+            AVFoundation.AVCaptureSession.setAutomaticallyRunsDeferredStart_, 0
+        )
