@@ -3,6 +3,10 @@ from PyObjCTools.TestSupport import TestCase, min_os_level
 
 
 class TestAVMetadataFormat(TestCase):
+    def test_typed_enum(self):
+        self.assertIsTypedEnum(AVFoundation.AVMetadataFormat, str)
+        self.assertIsTypedEnum(AVFoundation.AVMetadataKey, str)
+
     @min_os_level("10.7")
     def testConstants10_7(self):
         self.assertIsInstance(AVFoundation.AVMetadataKeySpaceCommon, str)
@@ -451,8 +455,10 @@ class TestAVMetadataFormat(TestCase):
         self.assertIsInstance(AVFoundation.AVMetadataID3MetadataKeyCommercial, str)
         self.assertIsInstance(AVFoundation.AVMetadataExtraAttributeInfoKey, str)
 
-    @min_os_level("10.11.3")
+    @min_os_level("10.12")
     def testConstants10_11_3(self):
+        # The documentation says this value is availaable on 10.11.3, but testing
+        # shows the value isn't available in 10.11.6.
         self.assertIsInstance(AVFoundation.AVMetadataKeySpaceHLSDateRange, str)
 
     @min_os_level("10.12")
@@ -463,3 +469,65 @@ class TestAVMetadataFormat(TestCase):
     def testConstants10_13(self):
         self.assertIsInstance(AVFoundation.AVMetadataKeySpaceAudioFile, str)
         self.assertIsInstance(AVFoundation.AVMetadataFormatUnknown, str)
+
+    @min_os_level("11.0")
+    def testConstants11_0(self):
+        self.assertIsInstance(
+            AVFoundation.AVMetadataCommonKeyAccessibilityDescription, str
+        )
+        self.assertIsInstance(
+            AVFoundation.AVMetadataQuickTimeUserDataKeyAccessibilityDescription, str
+        )
+        self.assertIsInstance(
+            AVFoundation.AVMetadataISOUserDataKeyAccessibilityDescription, str
+        )
+        self.assertIsInstance(
+            AVFoundation.AVMetadataQuickTimeMetadataKeyAccessibilityDescription, str
+        )
+
+    @min_os_level("12.0")
+    def testConstants12_0(self):
+        self.assertIsInstance(AVFoundation.AVMetadataQuickTimeMetadataKeyIsMontage, str)
+
+    @min_os_level("15.0")
+    def testConstants15_0(self):
+        self.assertIsInstance(
+            AVFoundation.AVMetadataQuickTimeMetadataKeyFullFrameRatePlaybackIntent, str
+        )
+
+    @min_os_level("26.0")
+    def testConstants26_0(self):
+        self.assertIsInstance(
+            AVFoundation.AVMetadataQuickTimeMetadataKeyCinematicVideoIntent, str
+        )
+        self.assertIsInstance(
+            AVFoundation.AVMetadataQuickTimeMetadataKeyCameraLensModel, str
+        )
+        self.assertIsInstance(
+            AVFoundation.AVMetadataQuickTimeMetadataKeyCameraFocalLength35mmEquivalent,
+            str,
+        )
+
+        self.assertIsInstance(
+            AVFoundation.AVMetadataQuickTimeMetadataKeyCameraISOSensitivity, str
+        )
+        self.assertIsInstance(
+            AVFoundation.AVMetadataQuickTimeMetadataKeyCameraWhiteBalance, str
+        )
+        self.assertIsInstance(
+            AVFoundation.AVMetadataQuickTimeMetadataKeyWhiteBalanceByCCTWhiteBalanceFactors,
+            str,
+        )
+        self.assertIsInstance(
+            AVFoundation.AVMetadataQuickTimeMetadataKeyWhiteBalanceByCCTColorMatrices,
+            str,
+        )
+        self.assertIsInstance(
+            AVFoundation.AVMetadataQuickTimeMetadataKeyCameraShutterSpeedAngle, str
+        )
+        self.assertIsInstance(
+            AVFoundation.AVMetadataQuickTimeMetadataKeyCameraShutterSpeedTime, str
+        )
+        self.assertIsInstance(
+            AVFoundation.AVMetadataQuickTimeMetadataKeyCameraLensIrisFNumber, str
+        )
