@@ -14,7 +14,7 @@ sys.path.insert(0, os.path.dirname(__file__))
 
 from pyobjc_setup import setup, Extension  # noqa: E402
 
-VERSION = "10.3.2"
+VERSION = "12.1"
 
 setup(
     name="pyobjc-framework-AVRouting",
@@ -26,7 +26,6 @@ setup(
             "AVRouting._AVRouting",
             ["Modules/_AVRouting.m"],
             extra_link_args=["-framework", "AVRouting"],
-            py_limited_api=True,
             depends=[
                 os.path.join("Modules", fn)
                 for fn in os.listdir("Modules")
@@ -40,5 +39,4 @@ setup(
         "pyobjc-framework-Cocoa>=" + VERSION,
     ],
     long_description=__doc__,
-    options={"bdist_wheel": {"py_limited_api": "cp36"}},
 )
