@@ -1,4 +1,8 @@
 #!/usr/bin/python
+"""
+This script is a basic example of how to use Key Value Observation
+in Python
+"""
 
 import objc
 from Foundation import NSKeyValueChangeNewKey, NSKeyValueObservingOptionNew, NSObject
@@ -7,10 +11,9 @@ from Foundation import NSKeyValueChangeNewKey, NSKeyValueObservingOptionNew, NSO
 class MyClass(NSObject):
     base = objc.ivar("base", objc._C_INT)
     power = objc.ivar("power", objc._C_INT)
-    result = objc.ivar("result", objc._C_INT)
 
     def result(self):
-        return self.base ** self.power
+        return self.base**self.power
 
 
 MyClass.setKeys_triggerChangeNotificationsForDependentKey_(["base", "power"], "result")
