@@ -42,32 +42,59 @@ static PyMethodDef mod_methods[] = {
     {0, 0, 0, 0} /* sentinel */
 };
 
-/* Python glue */
+static int
+mod_exec_module(PyObject* m)
+{
+    if (PyObjC_ImportAPI(m) == -1)
+        return -1;
+    if (init_audio_buffer(m) == -1)
+        return -1;
+    if (init_audio_buffer_list(m) == -1)
+        return -1;
+    if (init_audio_value_translation(m) == -1)
+        return -1;
+    if (init_audio_channel_description(m) == -1)
+        return -1;
+    if (init_audio_channel_layout(m) == -1)
+        return -1;
+
+    return 0;
+}
+
+static struct PyModuleDef_Slot mod_slots[] = {
+    {.slot = Py_mod_exec, .value = (void*)mod_exec_module},
+#if PY_VERSION_HEX >= 0x030c0000
+    {
+        .slot  = Py_mod_multiple_interpreters,
+        .value = Py_MOD_MULTIPLE_INTERPRETERS_NOT_SUPPORTED,
+    },
+#endif
+#if PY_VERSION_HEX >= 0x030d0000
+    {
+        .slot  = Py_mod_gil,
+        .value = Py_MOD_GIL_NOT_USED,
+    },
+#endif
+    {/* Sentinel */
+     .slot  = 0,
+     .value = 0}};
+
 static struct PyModuleDef mod_module = {
-    PyModuleDef_HEAD_INIT, "_CoreAudio", NULL, 0, mod_methods, NULL, NULL, NULL, NULL};
+    .m_base     = PyModuleDef_HEAD_INIT,
+    .m_name     = "_CoreAudio",
+    .m_doc      = NULL,
+    .m_size     = 0,
+    .m_methods  = mod_methods,
+    .m_slots    = mod_slots,
+    .m_traverse = NULL,
+    .m_clear    = NULL,
+    .m_free     = NULL,
+};
 
 PyObject* PyInit__CoreAudio(void);
 
-PyObject* __attribute__((__visibility__("default"))) PyInit__CoreAudio(void)
+PyObject* __attribute__((__visibility__("default")))
+PyInit__CoreAudio(void)
 {
-    PyObject* m;
-    m = PyModule_Create(&mod_module);
-    if (!m) {
-        return NULL;
-    }
-
-    if (PyObjC_ImportAPI(m) == -1)
-        return NULL;
-    if (init_audio_buffer(m) == -1)
-        return NULL;
-    if (init_audio_buffer_list(m) == -1)
-        return NULL;
-    if (init_audio_value_translation(m) == -1)
-        return NULL;
-    if (init_audio_channel_description(m) == -1)
-        return NULL;
-    if (init_audio_channel_layout(m) == -1)
-        return NULL;
-
-    return m;
+    return PyModuleDef_Init(&mod_module);
 }
