@@ -19,7 +19,7 @@ sys.path.insert(0, os.path.dirname(__file__))
 
 from pyobjc_setup import Extension, setup  # noqa: E402
 
-VERSION = "10.3.2"
+VERSION = "12.1"
 
 
 distutils.unixccompiler.UnixCCompiler.src_extensions.append(".mm")
@@ -33,13 +33,11 @@ setup(
             "CoreAudio._inlines",
             ["Modules/_CoreAudio_inlines.mm"],
             extra_link_args=["-framework", "CoreAudio"],
-            py_limited_api=True,
         ),
         Extension(
             "CoreAudio._CoreAudio",
             ["Modules/_CoreAudio.m"],
             extra_link_args=["-framework", "CoreAudio"],
-            # py_limited_api=True,
             depends=[
                 os.path.join("Modules", fn)
                 for fn in os.listdir("Modules")
@@ -50,5 +48,4 @@ setup(
     version=VERSION,
     install_requires=["pyobjc-core>=" + VERSION, "pyobjc-framework-Cocoa>=" + VERSION],
     long_description=__doc__,
-    #    options={"bdist_wheel": {"py_limited_api": "cp36"}},
 )
