@@ -15,7 +15,7 @@ sys.path.insert(0, os.path.dirname(__file__))
 
 from pyobjc_setup import setup, Extension  # noqa: E402
 
-VERSION = "10.3.2"
+VERSION = "12.1"
 
 setup(
     name="pyobjc-framework-CoreMIDI",
@@ -26,13 +26,11 @@ setup(
             "CoreMIDI._inlines",
             ["Modules/_CoreMIDI_inlines.m"],
             extra_link_args=["-framework", "CoreMIDI"],
-            py_limited_api=True,
         ),
         Extension(
             "CoreMIDI._CoreMIDI",
             ["Modules/_CoreMIDI.m"],
             extra_link_args=["-framework", "CoreMIDI"],
-            py_limited_api=True,
             depends=[
                 os.path.join("Modules", fn)
                 for fn in os.listdir("Modules")
@@ -43,5 +41,4 @@ setup(
     version=VERSION,
     install_requires=["pyobjc-core>=" + VERSION, "pyobjc-framework-Cocoa>=" + VERSION],
     long_description=__doc__,
-    options={"bdist_wheel": {"py_limited_api": "cp36"}},
 )
