@@ -15,7 +15,7 @@ sys.path.insert(0, os.path.dirname(__file__))
 
 from pyobjc_setup import Extension, setup  # noqa: E402
 
-VERSION = "10.3.2"
+VERSION = "12.1"
 
 setup(
     name="pyobjc-framework-CoreServices",
@@ -33,7 +33,6 @@ setup(
             "CoreServices._inlines",
             ["Modules/_CoreServices_inlines.m"],
             extra_link_args=["-framework", "CoreServices"],
-            py_limited_api=True,
         )
     ],
     version=VERSION,
@@ -43,5 +42,4 @@ setup(
         "pyobjc-framework-Cocoa>=" + VERSION,
     ],
     long_description=__doc__,
-    options={"bdist_wheel": {"py_limited_api": "cp36"}},
 )
