@@ -473,6 +473,13 @@ class TestCTFont(TestCase):
     def testFunctions10_9(self):
         self.assertResultIsCFRetained(CoreText.CTFontCreateForStringWithLanguage)
 
+    @min_os_level("15.0")
+    def testFunctions15_0(self):
+        self.assertResultIsCFRetained(CoreText.CTFontCopyDefaultCascadeListForLanguages)
+        CoreText.CTFontGetTypographicBoundsForAdaptiveImageProvider
+        CoreText.CTFontDrawImageFromAdaptiveImageProviderAtPoint
+        CoreText.CTFontHasTable
+
     @min_os_level("10.10")
     def testConstants10_10(self):
         self.assertIsInstance(CoreText.kCTFontOpenTypeFeatureTag, str)
