@@ -3,14 +3,21 @@ Wrappers for the "CoreText" framework on macOS 10.5 or later. Core Text is an
 advanced, low-level technology for laying out text and handling fonts. It is
 designed for high performance and ease of use.
 
-These wrappers don't include documentation, please check Apple's documention
+These wrappers don't include documentation, please check Apple's documentation
 for information on how to use this framework and PyObjC's documentation
 for general tips and tricks regarding the translation between Python
 and (Objective-)C frameworks
 """
-from pyobjc_setup import Extension, setup
 
-VERSION = "6.2.2"
+import os
+import sys
+
+sys.path.insert(0, os.path.dirname(__file__))
+
+
+from pyobjc_setup import Extension, setup  # noqa: E402
+
+VERSION = "12.1"
 
 setup(
     name="pyobjc-framework-CoreText",
@@ -22,7 +29,6 @@ setup(
             "CoreText._manual",
             ["Modules/_manual.m"],
             extra_link_args=["-framework", "CoreServices"],
-            py_limited_api=True,
         )
     ],
     version=VERSION,
@@ -32,5 +38,4 @@ setup(
         "pyobjc-framework-Quartz>=" + VERSION,
     ],
     long_description=__doc__,
-    options={"bdist_wheel": {"py_limited_api": "cp36"}},
 )
