@@ -17,18 +17,15 @@ sys.path.insert(0, os.path.dirname(__file__))
 
 from pyobjc_setup import Extension, setup  # noqa: E402
 
-VERSION = "10.3.2"
+VERSION = "12.1"
 
 setup(
     name="pyobjc-framework-FSEvents",
     description="Wrappers for the framework FSEvents on macOS",
     min_os_level="10.5",
     packages=["FSEvents"],
-    ext_modules=[
-        Extension("FSEvents._callbacks", ["Modules/_callbacks.m"], py_limited_api=True)
-    ],
+    ext_modules=[Extension("FSEvents._callbacks", ["Modules/_callbacks.m"])],
     version=VERSION,
     install_requires=["pyobjc-core>=" + VERSION, "pyobjc-framework-Cocoa>=" + VERSION],
     long_description=__doc__,
-    options={"bdist_wheel": {"py_limited_api": "cp36"}},
 )
