@@ -1,9 +1,9 @@
 """
 Wrappers for "InstantMessage" framework on macOS 10.5 or later. This framework
 allows you to access iChat information, as well as a way to provide an
-auxilliary video source to iChat Theater.
+auxiliary video source to iChat Theater.
 
-These wrappers don't include documentation, please check Apple's documention
+These wrappers don't include documentation, please check Apple's documentation
 for information on how to use this framework and PyObjC's documentation
 for general tips and tricks regarding the translation between Python
 and (Objective-)C frameworks
@@ -11,9 +11,16 @@ and (Objective-)C frameworks
 Note that this framework is deprecated in OSX 10.9, use the Social framework
 instead if you target that OSX release.
 """
-from pyobjc_setup import setup
 
-VERSION = "6.2.2"
+import os
+import sys
+
+sys.path.insert(0, os.path.dirname(__file__))
+
+
+from pyobjc_setup import setup  # noqa: E402
+
+VERSION = "12.1"
 
 setup(
     name="pyobjc-framework-InstantMessage",
