@@ -15,7 +15,7 @@ sys.path.insert(0, os.path.dirname(__file__))
 
 from pyobjc_setup import setup, Extension  # noqa: E402  # noqa: E402
 
-VERSION = "10.3.2"
+VERSION = "12.1"
 
 setup(
     name="pyobjc-framework-IntentsUI",
@@ -27,7 +27,6 @@ setup(
             "IntentsUI._IntentsUI",
             ["Modules/_IntentsUI.m"],
             extra_link_args=["-framework", "IntentsUI"],
-            py_limited_api=True,
             depends=[
                 os.path.join("Modules", fn)
                 for fn in os.listdir("Modules")
