@@ -1,6 +1,3 @@
-#ifndef PyObjC_GIL_DISABLED
-#define Py_LIMITED_API 0x03060000
-#endif
 #define PY_SSIZE_T_CLEAN
 #include "Python.h"
 #include "pyobjc-api.h"
@@ -50,25 +47,56 @@ static PyMethodDef mod_methods[] = {
     {0, 0, 0, 0} /* sentinel */
 };
 
-/* Python glue */
+static int
+mod_exec_module(PyObject* m)
+{
+    if (PyModule_AddObject(m, "_inline_list_", PyObjC_CreateInlineTab(function_map))
+        < 0) {
+        return -1;
+    }
+
+    return 0;
+}
+
+static struct PyModuleDef_Slot mod_slots[] = {
+    {.slot = Py_mod_exec, .value = (void*)mod_exec_module},
+#if PY_VERSION_HEX >= 0x030c0000
+    {
+        /* This extension does not use the CPython API other than initializing
+         * the module, hence is safe with subinterpreters and per-interpreter
+         * GILs
+         */
+        .slot  = Py_mod_multiple_interpreters,
+        .value = Py_MOD_PER_INTERPRETER_GIL_SUPPORTED,
+    },
+#endif
+#if PY_VERSION_HEX >= 0x030d0000
+    {
+        /* The code in this extension should be safe to use without the GIL */
+        .slot  = Py_mod_gil,
+        .value = Py_MOD_GIL_NOT_USED,
+    },
+#endif
+    {/* Sentinel */
+     .slot  = 0,
+     .value = 0}};
+
 static struct PyModuleDef mod_module = {
-    PyModuleDef_HEAD_INIT, "_inlines", NULL, 0, mod_methods, NULL, NULL, NULL, NULL};
+    .m_base     = PyModuleDef_HEAD_INIT,
+    .m_name     = "_inlines",
+    .m_doc      = NULL,
+    .m_size     = 0,
+    .m_methods  = mod_methods,
+    .m_slots    = mod_slots,
+    .m_traverse = NULL,
+    .m_clear    = NULL,
+    .m_free     = NULL,
+};
 
 PyObject* PyInit__inlines(void);
 
-PyObject*
+PyObject* __attribute__((__visibility__("default")))
 PyInit__inlines(void)
 {
-    PyObject* m;
-    m = PyModule_Create(&mod_module);
-    if (!m) {
-        return NULL;
-    }
-
-    if (PyModule_AddObject(m, "_inline_list_", PyObjC_CreateInlineTab(function_map))
-        < 0) {
-        return NULL;
-    }
-
-    return m;
+    return PyModuleDef_Init(&mod_module);
 }
