@@ -15,7 +15,7 @@ sys.path.insert(0, os.path.dirname(__file__))
 
 from pyobjc_setup import Extension, setup  # noqa: E402
 
-VERSION = "10.3.2"
+VERSION = "12.1"
 
 setup(
     name="pyobjc-framework-Metal",
@@ -27,13 +27,11 @@ setup(
             "Metal._inlines",
             ["Modules/_Metal_inlines.m"],
             extra_link_args=["-framework", "Metal"],
-            py_limited_api=True,
         ),
         Extension(
             "Metal._Metal",
             ["Modules/_Metal.m"],
             extra_link_args=["-framework", "Metal"],
-            py_limited_api=True,
             depends=[
                 os.path.join("Modules", fn)
                 for fn in os.listdir("Modules")
@@ -44,5 +42,4 @@ setup(
     version=VERSION,
     install_requires=["pyobjc-core>=" + VERSION, "pyobjc-framework-Cocoa>=" + VERSION],
     long_description=__doc__,
-    options={"bdist_wheel": {"py_limited_api": "cp36"}},
 )
