@@ -15,19 +15,18 @@ sys.path.insert(0, os.path.dirname(__file__))
 
 from pyobjc_setup import Extension, setup  # noqa: E402
 
-VERSION = "10.3.2"
+VERSION = "12.1"
 
 setup(
     name="pyobjc-framework-MetricKit",
     description="Wrappers for the framework MetricKit on macOS",
-    min_os_level="10.16",
+    min_os_level="11.0",
     packages=["MetricKit"],
     ext_modules=[
         Extension(
             "MetricKit._MetricKit",
             ["Modules/_MetricKit.m"],
             extra_link_args=["-framework", "MetricKit"],
-            py_limited_api=True,
             depends=[
                 os.path.join("Modules", fn)
                 for fn in os.listdir("Modules")
