@@ -14,7 +14,7 @@ sys.path.insert(0, os.path.dirname(__file__))
 
 from pyobjc_setup import Extension, setup  # noqa: E402
 
-VERSION = "10.3.2"
+VERSION = "12.1"
 
 setup(
     name="pyobjc-framework-Network",
@@ -26,11 +26,9 @@ setup(
             "Network._Network",
             ["Modules/_Network.m"],
             extra_link_args=["-framework", "Network"],
-            py_limited_api=True,
         )
     ],
     version=VERSION,
     install_requires=["pyobjc-core>=" + VERSION, "pyobjc-framework-Cocoa>=" + VERSION],
     long_description=__doc__,
-    options={"bdist_wheel": {"py_limited_api": "cp36"}},
 )
