@@ -15,7 +15,7 @@ sys.path.insert(0, os.path.dirname(__file__))
 
 from pyobjc_setup import setup  # noqa: E402
 
-VERSION = "10.3.2"
+VERSION = "12.1"
 
 setup(
     name="pyobjc-framework-PHASE",
@@ -28,5 +28,4 @@ setup(
         "pyobjc-framework-AVFoundation>=" + VERSION,
     ],
     long_description=__doc__,
-    options={"bdist_wheel": {"py_limited_api": "cp36"}},
 )
