@@ -15,19 +15,18 @@ sys.path.insert(0, os.path.dirname(__file__))
 
 from pyobjc_setup import setup, Extension  # noqa: E402
 
-VERSION = "10.3.2"
+VERSION = "12.1"
 
 setup(
     name="pyobjc-framework-PassKit",
     description="Wrappers for the framework PassKit on macOS",
-    min_os_level="10.16",
+    min_os_level="11.0",
     packages=["PassKit"],
     ext_modules=[
         Extension(
             "PassKit._PassKit",
             ["Modules/_PassKit.m"],
             extra_link_args=["-framework", "PassKit"],
-            py_limited_api=True,
             depends=[
                 os.path.join("Modules", fn)
                 for fn in os.listdir("Modules")
@@ -38,5 +37,4 @@ setup(
     version=VERSION,
     install_requires=["pyobjc-core>=" + VERSION, "pyobjc-framework-Cocoa>=" + VERSION],
     long_description=__doc__,
-    options={"bdist_wheel": {"py_limited_api": "cp36"}},
 )
