@@ -878,6 +878,8 @@ class TestCGImageProperties(TestCase):
         self.assertIsInstance(Quartz.kIIOCameraModelType_GenericPinhole, str)
         self.assertIsInstance(Quartz.kIIOCameraModel_Intrinsics, str)
 
+        self.assertIsInstance(Quartz.kCGImagePropertyOpenEXRCompression, str)
+
     @min_os_level("13.3")
     def testConstants13_3(self):
         # kCGImagePropertyOpenEXRCompression is no longer present in the 14.0 SDK,
@@ -889,3 +891,24 @@ class TestCGImageProperties(TestCase):
     def testConstants14_4(self):
         self.assertIsInstance(Quartz.kCGImagePropertyTIFFXPosition, str)
         self.assertIsInstance(Quartz.kCGImagePropertyTIFFYPosition, str)
+
+    @min_os_level("15.0")
+    def testConstants15_0(self):
+        self.assertIsInstance(Quartz.kCGImageAuxiliaryDataTypeISOGainMap, str)
+        self.assertIsInstance(Quartz.kCGImageAuxiliaryDataInfoColorSpace, str)
+        self.assertIsInstance(Quartz.kCGImagePropertyGroupImageIndexMonoscopic, str)
+        self.assertIsInstance(Quartz.kCGImagePropertyGroupImageIsMonoscopicImage, str)
+        self.assertIsInstance(Quartz.kCGImagePropertyGroupImageStereoAggressors, str)
+        self.assertIsInstance(Quartz.kIIOStereoAggressors_Type, str)
+        self.assertIsInstance(Quartz.kIIOStereoAggressors_SubTypeURI, str)
+        self.assertIsInstance(Quartz.kIIOStereoAggressors_Severity, str)
+        self.assertIsInstance(Quartz.kCGImagePropertyGroupMonoscopicImageLocation, str)
+        self.assertIsInstance(Quartz.kIIOMonoscopicImageLocation_Unspecified, str)
+        self.assertIsInstance(Quartz.kIIOMonoscopicImageLocation_Left, str)
+        self.assertIsInstance(Quartz.kIIOMonoscopicImageLocation_Right, str)
+        self.assertIsInstance(Quartz.kIIOMonoscopicImageLocation_Center, str)
+
+    @min_os_level("26.0")
+    def testConstants26_0(self):
+        self.assertIsInstance(Quartz.kCGImageProviderPreferredTileWidth, str)
+        self.assertIsInstance(Quartz.kCGImageProviderPreferredTileHeight, str)
