@@ -16,7 +16,7 @@ sys.path.insert(0, os.path.dirname(__file__))
 
 from pyobjc_setup import Extension, setup  # noqa: E402
 
-VERSION = "10.3.2"
+VERSION = "12.1"
 
 setup(
     name="pyobjc-framework-ScreenSaver",
@@ -27,11 +27,9 @@ setup(
             "ScreenSaver._inlines",
             ["Modules/_ScreenSaver_inlines.m"],
             extra_link_args=["-framework", "ScreenSaver"],
-            py_limited_api=True,
         )
     ],
     version=VERSION,
     install_requires=["pyobjc-core>=" + VERSION, "pyobjc-framework-Cocoa>=" + VERSION],
     long_description=__doc__,
-    options={"bdist_wheel": {"py_limited_api": "cp36"}},
 )
