@@ -15,7 +15,7 @@ sys.path.insert(0, os.path.dirname(__file__))
 
 from pyobjc_setup import Extension, setup  # noqa: E402
 
-VERSION = "10.3.2"
+VERSION = "12.1"
 
 setup(
     name="pyobjc-framework-SecurityInterface",
@@ -26,7 +26,6 @@ setup(
             "SecurityInterface._SecurityInterface",
             ["Modules/_SecurityInterface.m"],
             extra_link_args=["-framework", "SecurityInterface"],
-            py_limited_api=True,
         )
     ],
     version=VERSION,
@@ -36,5 +35,4 @@ setup(
         "pyobjc-framework-Security>=" + VERSION,
     ],
     long_description=__doc__,
-    options={"bdist_wheel": {"py_limited_api": "cp36"}},
 )
