@@ -15,7 +15,7 @@ sys.path.insert(0, os.path.dirname(__file__))
 
 from pyobjc_setup import Extension, setup  # noqa: E402
 
-VERSION = "10.3.2"
+VERSION = "12.1"
 
 setup(
     name="pyobjc-framework-SharedWithYou",
@@ -27,7 +27,6 @@ setup(
             "SharedWithYou._SharedWithYou",
             ["Modules/_SharedWithYou.m"],
             extra_link_args=["-framework", "SharedWithYou"],
-            py_limited_api=True,
             depends=[
                 os.path.join("Modules", fn)
                 for fn in os.listdir("Modules")
@@ -41,5 +40,4 @@ setup(
         "pyobjc-framework-SharedWithYouCore>=" + VERSION,
     ],
     long_description=__doc__,
-    options={"bdist_wheel": {"py_limited_api": "cp36"}},
 )
