@@ -21,7 +21,7 @@ sys.path.insert(0, os.path.dirname(__file__))
 
 from pyobjc_setup import Extension, setup  # noqa: E402
 
-VERSION = "10.3.2"
+VERSION = "12.1"
 
 setup(
     name="pyobjc-framework-SyncServices",
@@ -32,7 +32,6 @@ setup(
             "SyncServices._SyncServices",
             ["Modules/_SyncServices.m"],
             extra_link_args=["-framework", "SyncServices"],
-            py_limited_api=True,
             depends=[
                 os.path.join("Modules", fn)
                 for fn in os.listdir("Modules")
@@ -47,5 +46,4 @@ setup(
         "pyobjc-framework-CoreData>=" + VERSION,
     ],
     long_description=__doc__,
-    options={"bdist_wheel": {"py_limited_api": "cp36"}},
 )
