@@ -15,7 +15,7 @@ sys.path.insert(0, os.path.dirname(__file__))
 
 from pyobjc_setup import Extension, setup  # noqa: E402
 
-VERSION = "10.3.2"
+VERSION = "12.1"
 
 setup(
     name="pyobjc-framework-libdispatch",
@@ -26,12 +26,10 @@ setup(
         Extension(
             "dispatch._inlines",
             ["Modules/_libdispatch_inlines.m"],
-            py_limited_api=True,
         ),
         Extension(
             "dispatch._dispatch",
             ["Modules/_libdispatch.m"],
-            # py_limited_api=True
         ),
     ],
     version=VERSION,
