@@ -13,7 +13,7 @@ import re
 from setuptools import setup, Command
 from setuptools.command import egg_info
 
-VERSION = "10.3.2"
+VERSION = "12.1"
 
 # Table with all framework wrappers and the OSX releases where they are
 # first supported, and where support was removed. The introduced column
@@ -28,6 +28,7 @@ FRAMEWORK_WRAPPERS = [
     ("AdServices", "11.0", None),
     ("AdSupport", "10.14", None),
     ("AppTrackingTransparency", "11.0", None),
+    ("ARKit", "26.0", None),
     ("AudioVideoBridging", "10.8", None),
     ("AuthenticationServices", "10.15", None),
     ("AutomaticAssessmentConfiguration", "10.15", None),
@@ -46,10 +47,12 @@ FRAMEWORK_WRAPPERS = [
     ("CFNetwork", None, None),
     ("CalendarStore", "10.5", None),
     ("CallKit", "11.0", None),
+    ("Carbon", None, None),
     ("Cinematic", "14.0", None),
     ("ClassKit", "11.0", None),
     ("CloudKit", "10.10", None),
     ("Cocoa", None, None),
+    ("CompositorServices", "26.0", None),
     ("Collaboration", "10.5", None),
     ("ColorSync", "10.13", None),
     ("Contacts", "10.11", None),
@@ -72,6 +75,7 @@ FRAMEWORK_WRAPPERS = [
     ("CryptoTokenKit", "10.10", None),
     ("DataDetection", "12.0", None),
     ("DeviceCheck", "10.15", None),
+    ("DeviceDiscoveryExtension", "15.0", None),
     ("DictionaryServices", "10.5", None),
     ("DiscRecording", None, None),
     ("DiscRecordingUI", None, None),
@@ -85,9 +89,11 @@ FRAMEWORK_WRAPPERS = [
     ("FileProvider", "10.15", None),
     ("FileProviderUI", "10.15", None),
     ("FSEvents", "10.5", None),
+    ("FSKit", "15.4", None),
     ("FinderSync", "10.10", None),
     ("GameCenter", "10.8", None),
     ("GameController", "10.9", None),
+    ("GameSave", "26.0", None),
     ("HealthKit", "13.0", None),
     ("InputMethodKit", "10.5", None),
     ("ImageCaptureCore", "10.6", None),
@@ -107,6 +113,7 @@ FRAMEWORK_WRAPPERS = [
     ("MailKit", "12.0", None),
     ("MapKit", "10.9", None),
     ("MediaAccessibility", "10.9", None),
+    ("MediaExtension", "15.0", None),
     ("MediaLibrary", "10.9", None),
     ("MediaPlayer", "10.12", None),
     ("MediaToolbox", "10.9", None),
@@ -146,6 +153,7 @@ FRAMEWORK_WRAPPERS = [
     ("Security", None, None),
     ("SecurityFoundation", None, None),
     ("SecurityInterface", None, None),
+    ("SecurityUI", "15.4", None),
     ("SearchKit", None, None),
     ("ServiceManagement", "10.6", None),
     ("ShazamKit", "12.0", None),
@@ -198,6 +206,11 @@ MACOS_TO_DARWIN = {
     "13.0": "22.0",
     "14.0": "23.0",
     "14.4": "23.4",
+    "15.0": "24.0",
+    "15.1": "24.1",
+    "15.2": "24.2",
+    "15.4": "24.4",
+    "26.0": "25.0",
 }
 
 
@@ -263,17 +276,17 @@ Development Status :: 5 - Production/Stable
 Environment :: Console
 Environment :: MacOS X :: Cocoa
 Intended Audience :: Developers
-License :: OSI Approved :: MIT License
 Natural Language :: English
 Operating System :: MacOS :: MacOS X
 Programming Language :: Python
 Programming Language :: Python :: 3
-Programming Language :: Python :: 3.8
-Programming Language :: Python :: 3.9
 Programming Language :: Python :: 3.10
 Programming Language :: Python :: 3.11
 Programming Language :: Python :: 3.12
 Programming Language :: Python :: 3.13
+Programming Language :: Python :: 3.14
+Programming Language :: Python :: 3.15
+Programming Language :: Python :: Free Threading :: 3 - Stable
 Programming Language :: Objective C
 Topic :: Software Development :: Libraries :: Python Modules
 Topic :: Software Development :: User Interfaces
@@ -333,7 +346,9 @@ def scan_info(path):
     packages = []
     required = []
 
-    a = compile(open(path).read(), path, "exec", ast.PyCF_ONLY_AST)
+    with open(path) as stream:
+        contents = stream.read()
+    a = compile(contents, path, "exec", ast.PyCF_ONLY_AST)
     for node in ast.walk(a):
         if isinstance(node, ast.Call) and getattr(node.func, "id", "") == "setup":
             for child in node.keywords:
@@ -354,10 +369,11 @@ def get_imports(prefix):
     for topdir, _dirnames, fnames in os.walk(prefix):
         for fn in fnames:
             if fn == "__init__.py":
-                for line in open(os.path.join(topdir, fn)).readlines():
-                    m = re.match(r"^\s+import\s([A-Za-z0-9]*)$", line)
-                    if m is not None:
-                        imports.add(m.group(1))
+                with open(os.path.join(topdir, fn)) as stream:
+                    for line in stream.readlines():
+                        m = re.match(r"^\s+import\s([A-Za-z0-9]*)$", line)
+                        if m is not None:
+                            imports.add(m.group(1))
 
     return imports
 
@@ -518,53 +534,55 @@ class oc_test(Command):
                     failures += 1
 
         print("  validating sdist archives...")
-        devnull = open("/dev/null", "a")
-        for nm in ("pyobjc", "pyobjc-core") + tuple(
-            sorted(nm for nm in os.listdir("..") if nm.startswith("pyobjc-framework-"))
-        ):
-            print(f"    {nm}")
-            subdir = os.path.join("..", nm)
-            if os.path.exists(os.path.join(subdir, "dist")):
-                shutil.rmtree(os.path.join(subdir, "dist"))
+        with open("/dev/null", "a") as devnull:
+            for nm in ("pyobjc", "pyobjc-core") + tuple(
+                sorted(
+                    nm for nm in os.listdir("..") if nm.startswith("pyobjc-framework-")
+                )
+            ):
+                print(f"    {nm}")
+                subdir = os.path.join("..", nm)
+                if os.path.exists(os.path.join(subdir, "dist")):
+                    shutil.rmtree(os.path.join(subdir, "dist"))
 
-            p = subprocess.check_call(
-                [sys.executable, "setup.py", "sdist"],
-                cwd=subdir,
-                stdout=devnull,
-                stderr=devnull,
-            )
-            files = glob.glob(os.path.join(subdir, "dist", "*.tar.gz"))
+                p = subprocess.check_call(
+                    [sys.executable, "setup.py", "sdist"],
+                    cwd=subdir,
+                    stdout=devnull,
+                    stderr=devnull,
+                )
+                files = glob.glob(os.path.join(subdir, "dist", "*.tar.gz"))
 
-            if not files:
-                print(f"No sdist in {nm}")
-                failures += 1
+                if not files:
+                    print(f"No sdist in {nm}")
+                    failures += 1
 
-            elif len(files) > 1:
-                print(f"Too many sdist in {nm}")
-                failures += 1
+                elif len(files) > 1:
+                    print(f"Too many sdist in {nm}")
+                    failures += 1
 
-            else:
-                t = tarfile.open(files[0], "r:gz")
-                for fn in t.getnames():
-                    if fn.startswith("/"):
-                        print(f"Absolute path in sdist for {nm}")
-                        failures += 1
-
-                    for p in (
-                        "__pycache__",
-                        ".pyc",
-                        ".pyo",
-                        ".so",
-                        ".dSYM",
-                        ".eggs",
-                        ".app",
-                        "/build/",
-                        "/dist/",
-                    ):
-                        if p in fn:
-                            print(f"Unwanted pattern {p!r} in sdist for {nm}: {fn}")
+                else:
+                    t = tarfile.open(files[0], "r:gz")
+                    for fn in t.getnames():
+                        if fn.startswith("/"):
+                            print(f"Absolute path in sdist for {nm}")
                             failures += 1
 
+                        for p in (
+                            "__pycache__",
+                            ".pyc",
+                            ".pyo",
+                            ".so",
+                            ".dSYM",
+                            ".eggs",
+                            ".app",
+                            "/build/",
+                            "/dist/",
+                        ):
+                            if p in fn:
+                                print(f"Unwanted pattern {p!r} in sdist for {nm}: {fn}")
+                                failures += 1
+
         print("  validating long description...")
         for nm in ("pyobjc", "pyobjc-core") + tuple(
             sorted(nm for nm in os.listdir("..") if nm.startswith("pyobjc-framework-"))
@@ -704,10 +722,10 @@ setup(
     extras_require={
         "allbindings": BASE_REQUIRES + framework_requires(include_all=True)
     },
-    python_requires=">=3.8",
+    python_requires=">=3.10",
     setup_requires=[],
     classifiers=CLASSIFIERS,
-    license="MIT License",
+    license="MIT",
     zip_safe=True,
     # workaround for setuptools 0.6b4 bug
     dependency_links=[],
